/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Threading;
using PaintableSimulator.Engine.PaintableComputers;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using PaintableSimulator.Engine.Resources;
using PaintableSimulator.Engine.PaintableComputers.Sensors;
using System.Reflection;

namespace PaintableSimulator.Engine
{
	/// <summary>
	/// 
	/// </summary>
	[Serializable()]
	public class LogEntry
	{
		/// <summary>
		/// 
		/// </summary>
		public DateTime TimeStamp;

		/// <summary>
		/// 
		/// </summary>
		public String Message;

		public LogEntry(String message)
		{
			TimeStamp = DateTime.Now;
			Message = message;
		}
	}


	/// <summary>
	/// 
	/// </summary>
	public class ThreadSynchronization
	{
		/// <summary>
		/// When true, thread shud finish its execution
		/// </summary>
		public bool DoFinish;

		/// <summary>
		/// When true, thread executes
		/// </summary>
		public bool NotPaused;

		/// <summary>
		/// 
		/// </summary>
		public ThreadSynchronization()
		{
			DoFinish = false;
			NotPaused = false;
		}
	}

    /// <summary>
    /// 
    /// </summary>
    public class SensorOnTheWall
    {
        public ISensor sensor;
        public Point location;

        public SensorOnTheWall(ISensor Sensor, Point Location)
        {
            sensor = Sensor;
            location = Location;
        }
    }

    /// <summary>
    /// 
    /// </summary>
    public class SensorEnviroment
    {
        /// <summary>
        /// 
        /// </summary>
        private EnvironmentDataFeeder envFeed = null;

        /// <summary>
        /// 
        /// </summary>
        private List<SensorOnTheWall> senzors = null;
        
        /// <summary>
        /// 
        /// </summary>
        public void Cycle(uint ticks)
        {
            for (int i = 0; i < senzors.Count; i++)
            {
                ISensor s = senzors[i].sensor;
                s.SetEnvironmentValue(envFeed.GetType(), envFeed.GetValue(senzors[i].location, ticks));
                ((GenericPaintableComputer)s).Cycle();
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="senzor"></param>
        public void AddSenzor(ISensor senzor, Point location)
        {
            senzors.Add(new SensorOnTheWall(senzor, location));
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="sensor"></param>
        public SensorEnviroment(Type sensor, Size wallSize)
        {
            envFeed = (EnvironmentDataFeeder)Activator.CreateInstance(sensor, new object[] { wallSize });
            senzors = new List<SensorOnTheWall>();
         }        
    }

	public class GenericSimulator : SimulatorBase
	{
		private EngineSettings simulationSettings;


		/// <summary>
		/// 
		/// </summary>
		private Queue<LogEntry> logging;

		/// <summary>
		/// 
		/// </summary>
		private SimulatorInfo info;


		/// <summary>
		/// 
		/// </summary>
		private Queue<RadioPacket> pendingRadioPackets = new Queue<RadioPacket>();

		/// <summary>
		/// 
		/// </summary>
		internal bool communicationEnabled;

		/// <summary>
		/// 
		/// </summary>
		public bool CommunicationEnabled
		{
			get { return communicationEnabled; }
			set { communicationEnabled = value; }
		}


		/// <summary>
		/// 
		/// </summary>
		private Thread simulatorThread;

		/// <summary>
		/// 
		/// </summary>
		private ThreadSynchronization simulatorThreadSynchronization;


		/// <summary>
		/// Initializes a new instance of the <see cref="T:GenericSimulator"/> class.
		/// </summary>
		public GenericSimulator()
		{
			// loads the default simulation settings values
			simulationSettings = new EngineSettings();

			logging = new Queue<LogEntry>();
			logging.Enqueue(new LogEntry(UserMessages.SimulatorCreation));

			if (log.IsInfoEnabled)
				log.Info(UserMessages.SimulatorCreation);

			info = new SimulatorInfo();

			// Objekt za sinhronizaciju
			simulatorThreadSynchronization = new ThreadSynchronization();

			//
			info.simulatorState = SimulatorStates.Created;
		}

		/// <summary>
		/// Destroys threadSimulatore and requests memory cleanup
		/// </summary>
		~GenericSimulator()
		{
			Terminate();
			logging.Enqueue(new LogEntry(UserMessages.SimulatorDestruction));
			if (log.IsInfoEnabled)
				log.Info(UserMessages.SimulatorDestruction);
			//
			info.simulatorState = SimulatorStates.None;
		}

		#region Thread stuff

		/// <summary>
		/// Runs this instance.
		/// </summary>
		public override void Run()
		{
			lock (simulatorThreadSynchronization)
			{
				//
				info.simulatorState = SimulatorStates.Running;

				simulatorThreadSynchronization.NotPaused = true;

				logging.Enqueue(new LogEntry(UserMessages.SimulatorRunning));
				if (log.IsInfoEnabled)
					log.Info(UserMessages.SimulatorRunning);

			}
		}

		/// <summary>
		/// Stops this instance.
		/// </summary>
		public override void Stop()
		{
			lock (simulatorThreadSynchronization)
			{
				//
				info.simulatorState = SimulatorStates.Stopped;

				simulatorThreadSynchronization.NotPaused = false;

				logging.Enqueue(new LogEntry(UserMessages.SimulatorPaused));
				if (log.IsInfoEnabled)
					log.Info(UserMessages.SimulatorPaused);

			}
		}


        /// <summary>
        /// 
        /// </summary>
        int PerformFaliure = 0;
        private void PerformOneStep()
        {
            //Faliure
            PerformFaliure--;
            if (PerformFaliure < 0)
            {
                RemoveRandomComputers(1);
                PerformFaliure = Convert.ToInt32(SimulationSettings.PaintableComputerFailureTime);
            }

            for (int i = 0; i < simulationSettings.SimulatorCycle; i++)
            {
                DoOneTick();
            }
            DoEnviromentFeed();
        }

		/// <summary>
		/// Steps this instance.
		/// </summary>
		public override void Step()
		{
			lock (simulatorThreadSynchronization)
			{
                info.simulatorState = SimulatorStates.Running;

                PerformOneStep();

				info.simulatorState = SimulatorStates.Stopped;
				logging.Enqueue(new LogEntry(UserMessages.SimlatorStepComplete));
				if (log.IsInfoEnabled)
					log.Info(UserMessages.SimlatorStepComplete);
			}
		}


		/// <summary>
		/// 
		/// </summary>
		private void ThreadProc()
		{
			// Deny via CAS malicious process fragments
			processFragmentPermissionSet.Deny();

			bool DoWork = true;
			// Petlja dok glavni program ne zahtjeva prekid programa
			do
			{
				if (simulatorThreadSynchronization.NotPaused)
				{
					lock (simulatorThreadSynchronization)
					{
                        PerformOneStep();
					}
				}
				// Da li treba prekinuti program
				DoWork = !simulatorThreadSynchronization.DoFinish;
				Thread.Sleep(0);
			} while (DoWork);
		}

		#endregion

		#region SimulatorBase Members

		/// <summary>
		/// 
		/// </summary>
		internal List<PConTheWall> computerList;

		/// <summary>
		/// Gets the computer list.
		/// </summary>
		/// <value>The computer list.</value>
		public override List<SimpleComputerInfo> ComputerList
		{
			get 
			{
				return null;
			}
		}

		/// <summary>
		/// Gets the computer list.
		/// </summary>
		/// <value>The computer list.</value>
		public List<PConTheWall> CustomComputerList
		{
			get { return computerList; }
		}


		/// <summary>
		/// Reads the log.
		/// </summary>
		/// <returns>
		/// The first LogEntry or null if there isn't any.
		/// </returns>
		public LogEntry ReadLog()
		{
			if (logging.Count > 0)
				return logging.Dequeue();
			else
				return null;
		}


		// provjerava da li je neka toka unutar radiusa neke druge toke
		/// <summary>
		/// Is point in radius of another point
		/// </summary>
		/// <param name="Center"></param>
		/// <param name="TestPoint"></param>
		/// <param name="Radius"></param>
		/// <returns></returns>
		private bool PointIsInRadius(Point Center, Point TestPoint, uint Radius)
		{
			int dx = Center.X - TestPoint.X;
			int dy = Center.Y - TestPoint.Y;
			int dr2 = dx * dx + dy * dy;

			return dr2 <= Radius * Radius;
		}


		/// <summary>
		/// Increments TickCount and
		/// - Deliveres radio packets
		/// - Runs scheduled Cycle methods of Paintable Computers
		/// - ...
		/// </summary>
		private void DoOneTick()
		{
			info.tickCount++;

			// Na poetku dostavimo pakete
			if (info.Communication.pendingPacketCount > 0)
			{
				RadioPacket rp = pendingRadioPackets.Peek();
				
				// ovo < je za svaki sluaj!
				while (rp.SendAfterTick <= info.tickCount)
				{
					PConTheWall originComputer = computerList[rp.OriginComputerIndex];

					pendingRadioPackets.Dequeue();
					for (int i = 0; i < originComputer.Neighbours.Count; i++)
					{
                        PConTheWall pc = originComputer.Neighbours[i];
						if (RadioPacket.WillBeSent(simulationSettings.RadioLinkFailurePercent))
						{
							pc.PaintableComputer.TransmissionAntenna = (NetworkPacket)rp.Payload;
							info.Communication.totalPacketCount++;
						}
						else
							info.Communication.lostPacketCount++;
					}

					// Ako se lista ispraznila, izai vani
					if (pendingRadioPackets.Count == 0)
						break;

					rp = pendingRadioPackets.Peek();
				}
				info.Communication.pendingPacketCount = Convert.ToUInt32(pendingRadioPackets.Count);
			}

            // Odradimo updateove
			for (int i = 0; i < info.computerCount; i++)
			{
				PConTheWall pc = computerList[i];

				// Ako je mrtav, zanemarimo ga...
				if (!(pc.IsAlive))
					continue;

				// Da li je PC na redu za obradu?
				if (pc.Tick <= info.tickCount)
				{
					// Neka odradi sto treba odraditi
					pc.PaintableComputer.Cycle();

					// Ako ima sto za poslati
					while (true)
					{
						NetworkPacket networkPacket = pc.PaintableComputer.TransmissionAntenna;
						if ((networkPacket != null) && (communicationEnabled))
						{
							RadioPacket rp = new RadioPacket();
							rp.OriginComputerIndex = i;
							rp.SendAfterTick = info.tickCount + simulationSettings.RadioLinkDelay;
							rp.Payload = networkPacket;
							pendingRadioPackets.Enqueue(rp);
						}
						else
							break;
					}

					// Odredimo kad ce se ponovo dogoditi odrada
					pc.Tick += Convert.ToUInt32(simulationSettings.SimulatorCycle +
												RandomNumberGenerator.GetRandomNumberRange(
													-simulationSettings.SimulatorCycleDeviation,
													simulationSettings.SimulatorCycleDeviation));
				}
				// Nije, idemo dalje...
			}
			info.Communication.pendingPacketCount = Convert.ToUInt32(pendingRadioPackets.Count);
		}

        //ToDo: KAKO KORISTITI TYpe A NE STRING?!?
        private Dictionary<String, SensorEnviroment> enviromentFeeders = new Dictionary<String, SensorEnviroment>();
        /// <summary>
		/// Registers the environment data feed.
		/// </summary>
		/// <param name="environmentDataFeed">The environment data feed.</param>
        public void RegisterEnvironmentDataFeed(Type environmentDataFeed)
        {
            if (!enviromentFeeders.ContainsKey(environmentDataFeed.Name))
            {
                enviromentFeeders.Add(environmentDataFeed.Name, new SensorEnviroment(environmentDataFeed, info.Environment.wallSize));
                logging.Enqueue(new LogEntry(String.Format(UserMessages.EnvironmentFeederRegistered, environmentDataFeed.Name)));
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private void DoEnviromentFeed()
        {
            foreach (SensorEnviroment se in enviromentFeeders.Values)
            {
                se.Cycle(info.tickCount);
            }
        }



        /// <summary>
        /// Calculates distance between 2 paintable computers
        /// </summary>
        /// <param name="pc1"></param>
        /// <param name="pc2"></param>
        /// <returns></returns>
        private static int ComputerDistance(PConTheWall pc1, PConTheWall pc2)
		{
			int dx = pc1.Location.X - pc2.Location.X;
			int dy = pc1.Location.Y - pc2.Location.Y;
			int dr2 = dx * dx + dy * dy;

			return dr2;
		}


        private Object InsertCumputerInList(Point Location, Type paintableComputerType, ref PConTheWall pc)
        {
			pc.Location = new Point(Location.X, Location.Y);

            Object o = Activator.CreateInstance(paintableComputerType,
                       new object[] { simulationSettings.MaximumNeighbourCount });
            
			
			pc.PaintableComputer = (PaintableComputerBase)o;
			pc.ID = pc.PaintableComputer.ComputerId;

			// Poetna boja
			pc.LastColor = Color.Empty;

			// iv je
			pc.IsAlive = true;

			// Upisi kad ce poceti s radom
			pc.Tick = Convert.ToUInt32(simulationSettings.SimulatorCycle +
									   RandomNumberGenerator.GetRandomNumberRange(-simulationSettings.SimulatorCycleDeviation,
																				  simulationSettings.SimulatorCycleDeviation));

			// Susjedi
            pc.Neighbours = new List<PConTheWall>();

			// Dodaj paintable u listu
			computerList.Add(pc);

            // Da li je kojim slucajem i senzor
            if (paintableComputerType.GetInterface(typeof(ISensor).Name) != null)
            {
                pc.IsSensor = true;
                foreach (Type feeder in ((ISensor)o).DataFeeders)
                {
                    try
                    {
                        SensorEnviroment se = enviromentFeeders[feeder.Name];
                        se.AddSenzor((ISensor)o, pc.Location);
                        logging.Enqueue(new LogEntry(String.Format(UserMessages.SensorAdded, ((FriendlyType)paintableComputerType).UserDefinedName, pc.Location.ToString())));
                    }
                    catch
                    {
                        logging.Enqueue(new LogEntry("Greka prilikom ubacivanja Senzora na lokaciju: " + Location.ToString()));
                    }
                }
            }
            return o;
        }


		/// <summary>
		/// Places Computers on wall
		/// </summary>
		/// <param name="maxComputerCount">Maximum computers on the wall</param>
		/// <param name="paintableComputerType">Type of the paintable computer.</param>
		private void SplashOnTheWall(uint maxComputerCount, Type paintableComputerType)
		{
			if (paintableComputerType == null)
				throw new ArgumentNullException("paintableComputerType");

			if (!paintableComputerType.IsSubclassOf(typeof(PaintableComputerBase)))
				throw new ArgumentException("The type of the paintableComputerType should inherit from PaintableComputerBase");

            // Aspect Ratio
            int xi = simulationSettings.WallSizeRatio.Width;
            int yi = simulationSettings.WallSizeRatio.Height;
            while ((xi * yi) < maxComputerCount)
            {
                xi += simulationSettings.WallSizeRatio.Width;
                yi += simulationSettings.WallSizeRatio.Height;
            }
            while ((xi * yi) > maxComputerCount)
            {
                xi--;
                yi = Convert.ToInt32((xi * simulationSettings.WallSizeRatio.Height) / simulationSettings.WallSizeRatio.Width);
            }

			info.computerCount = Convert.ToUInt32(xi * yi);

			// Varijacije na temu
			int Vartiations = Convert.ToInt32(simulationSettings.SpaceBetweenPaintableComputers / 2.2);


            //Wall Size
            info.Environment.wallSize.Width = Convert.ToInt32((xi+3) * simulationSettings.SpaceBetweenPaintableComputers);
            info.Environment.wallSize.Height = Convert.ToInt32((yi + 3) * simulationSettings.SpaceBetweenPaintableComputers);

            // Data Feeders
            logging.Enqueue(new LogEntry("Uitavam Enviroment Feeder objekte"));
            enviromentFeeders.Clear();
            foreach (FriendlyType ft in Plugins.ValidEnvironmentDataFeeders)
            {
                RegisterEnvironmentDataFeed(ft);
            }

			// Stvorimo listu PCova...
			computerList = new List<PConTheWall>();
			// ... i popunimo ih
            Point Location = new Point();
			for (int i = 1; i <= xi; i++)
				for (int j = 1; j <= yi; j++)
				{
        			// Male varijacije u slaganju
                    Location.X = Convert.ToInt32((i * simulationSettings.SpaceBetweenPaintableComputers) +
											RandomNumberGenerator.GetRandomNumberRange(-Vartiations, +Vartiations));
			        Location.Y = Convert.ToInt32((j * simulationSettings.SpaceBetweenPaintableComputers) +
											RandomNumberGenerator.GetRandomNumberRange(-Vartiations, +Vartiations));

                    PConTheWall pc = new PConTheWall();
                    InsertCumputerInList(Location, paintableComputerType, ref pc);
				}

		}

		#region Optimized Neighbour Search

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pc"></param>
        /// <param name="wall"></param>
        /// <param name="LeftRight"></param>
        /// <param name="WallBoundaries"></param>
        /// <param name="size"></param>
        /// <returns></returns>
        private uint DoCircle(PConTheWall pc,
                              PConTheWall[,] wall,
                              Point LeftRight,
                              Size WallBoundaries,
                              int size)
        {
            uint Return = Convert.ToUInt32(pc.Neighbours.Count);

            



            size--;
            if ((LeftRight.X >= 0) && (LeftRight.X < WallBoundaries.Height))
                for (int i = LeftRight.Y; i <= (LeftRight.Y + size); i++)
                    if ((i >= 0) && (i < WallBoundaries.Width))
                    {
                        if (wall[LeftRight.X, i] == null) continue;
                        if (!wall[LeftRight.X, i].IsAlive) continue;
                        if (PointIsInRadius(pc.Location, wall[LeftRight.X, i].Location,
                                            simulationSettings.WirelessRadius))
                            pc.Neighbours.Add(wall[LeftRight.X, i]);
                    }

            LeftRight.X += size;
            if ((LeftRight.X >= 0) && (LeftRight.X < WallBoundaries.Height))
                for (int i = LeftRight.Y; i <= (LeftRight.Y + size); i++)
                    if ((i >= 0) && (i < WallBoundaries.Width))
                    {
                        if (wall[LeftRight.X, i] == null) continue;
                        if (!wall[LeftRight.X, i].IsAlive) continue;
                        if (PointIsInRadius(pc.Location, wall[LeftRight.X, i].Location,
                                            simulationSettings.WirelessRadius))
                            pc.Neighbours.Add(wall[LeftRight.X, i]);
                    }

            LeftRight.X -= size;
            LeftRight.X++;
            size -= 2;
            if ((LeftRight.Y >= 0) && (LeftRight.Y < WallBoundaries.Width))
                for (int i = LeftRight.X; i <= (LeftRight.X + size); i++)
                    if ((i >= 0) && (i < WallBoundaries.Height))
                    {
                        if (wall[i, LeftRight.Y] == null) continue;
                        if (!wall[i, LeftRight.Y].IsAlive) continue;
                        if (PointIsInRadius(pc.Location, wall[i, LeftRight.Y].Location,
                                            simulationSettings.WirelessRadius))
                            pc.Neighbours.Add(wall[i, LeftRight.Y]);
                    }


            LeftRight.Y += size + 2;
            if ((LeftRight.Y >= 0) && (LeftRight.Y < WallBoundaries.Width))
                for (int i = LeftRight.X; i <= (LeftRight.X + size); i++)
                    if ((i >= 0) && (i < WallBoundaries.Height))
                    {
                        if (wall[i, LeftRight.Y] == null) continue;
                        if (!wall[i, LeftRight.Y].IsAlive) continue;
                        if (PointIsInRadius(pc.Location, wall[i, LeftRight.Y].Location,
                                            simulationSettings.WirelessRadius))
                            pc.Neighbours.Add(wall[i, LeftRight.Y]);
                    }

            return Convert.ToUInt32(pc.Neighbours.Count) - Return;
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="WidthAndHeight"></param>
        /// <returns></returns>
        public uint CalculateNeighbours(Size WidthAndHeight)
        {
            uint neigh = 0;

            // Napravimo reetku PCjeva
            PConTheWall[,] wall = new PConTheWall[WidthAndHeight.Height, WidthAndHeight.Width];
            //Najprije popuni sve null vrijednostima
            for (int i = 0; i < WidthAndHeight.Height; i++)
                for (int j = 0; j < WidthAndHeight.Width; j++)
                    wall[i, j] = null;

            //Sada popuni postojeim PCjevima
            for (int i = 0; i < computerList.Count; i++)
            {
                computerList[i].Neighbours.Clear();
                wall[computerList[i].Location.Y, computerList[i].Location.X] = computerList[i];
            }

            // Sad su u reetci
            // Krui oko njih dok god se 2 kruga ne pojavi niti jedan novi susjed
            Point LeftRight = new Point();
            uint neighbours = 0;
            for (int i = 0; i < computerList.Count; i++)
            {
                PConTheWall pc = computerList[i];

                // Da li je mrtav
                if (!pc.IsAlive)
                {
                    continue;
                }
                //Trenutni radius
                uint radius = 1;
                // Koliko ima PCona u redu
                int length = 3;
                // Gornji lijevi susjed
                Point StartPoint = new Point(pc.Location.Y, pc.Location.X);
                LeftRight.X = pc.Location.Y - 1;
                LeftRight.Y = pc.Location.X - 1;

                if (i == computerList.Count - 1)
                    radius = 1;

                // Kruimo poveavajui radijus do njegove granice
                uint NeighbourCount = 0;
                do
                {
                    NeighbourCount += DoCircle(pc, wall, LeftRight, WidthAndHeight, length);
                
                    // Novi gornji lijevim radius i duzina kvadrata
                    radius++;
                    length += 2;
                    LeftRight.X--;
                    LeftRight.Y--;

                } while ( (radius <= SimulationSettings.WirelessRadius) && (NeighbourCount < SimulationSettings.MaximumNeighbourCount));
                // Ako smo presli limit
                if (pc.Neighbours.Count > SimulationSettings.MaximumNeighbourCount)
                    pc.Neighbours.RemoveRange(Convert.ToInt32(SimulationSettings.MaximumNeighbourCount),
                                              Convert.ToInt32(pc.Neighbours.Count - SimulationSettings.MaximumNeighbourCount));
                neighbours += Convert.ToUInt32(pc.Neighbours.Count);
                
            }

#if DEBUG

			//// check for neighbourhood inconsitencies
			//// and maybe fix them
			//uint errors = 0;
			//for (int d = 0; d < computerList.Count; d++)
			//    for (int f = 0; f < computerList[d].Neighbours.Count; f++)
			//        if (!computerList[d].CompleteNeighbours[f].Neighbours.Contains(computerList[d]))
			//            errors++;

			//logging.Enqueue(new LogEntry(String.Format(UserMessages.NeighbourhoodInconsitencies, errors)));
			//if (log.IsInfoEnabled)
			//    log.Info(String.Format(UserMessages.NeighbourhoodInconsitencies, errors));

#endif

            logging.Enqueue(new LogEntry(String.Format(UserMessages.SimulatorNeighbourCount, neighbours)));
            return neighbours;
        }
        
        
        #endregion Optimized Neighbour Search

        /// <summary>
        /// 
        /// </summary>
        /// <param name="Locations"></param>
        /// <param name="Senzors"></param>
        public void InsertSensors(List<Point> Locations, List<Type> Senzors)
        {
            if (Locations.Count != Senzors.Count)
            {
                logging.Enqueue(new LogEntry("Broj lokacija i zenzora se ne poklapa!"));
                return;
            }
            for (int i = 0; i < Locations.Count; i++)
            {
                // Dodajemo Generic PC na zid
                PConTheWall pc = new PConTheWall();
                ISensor s = (ISensor)InsertCumputerInList(Locations[i], Senzors[i], ref pc);
            }
            if (Locations.Count>0)
                CalculateNeighbours(info.Environment.wallSize);
        }

		/// <summary>
		/// Initializes the simulator.
		/// </summary>
		public override void InitializeSimulator()
		{
			InitializeSimulator(typeof(GenericPaintableComputer));
		}


		/// <summary>
		/// Initializes the simulator with the specified paintable computer type.
		/// </summary>
		/// <param name="processFragmentType">Type of the process fragment.</param>
		public override void InitializeSimulator(Type paintableComputerType)
		{
			if (!paintableComputerType.IsSubclassOf(typeof(PaintableComputerBase)))
				throw new ArgumentException("paintableComputerType");

			logging.Enqueue(new LogEntry(UserMessages.SimulatorInitializationDefaults));
			if (log.IsInfoEnabled)
				log.Info(UserMessages.SimulatorInitializationDefaults);

			// Defaultne vrijenosti
			info.tickCount = 0;
			info.computerCount = 0;
			info.Communication.lostPacketCount = 0;
			info.Communication.pendingPacketCount = 0;
			info.Communication.totalPacketCount = 0;
			info.Environment.wallSize = new Size(0, 0);
            PerformFaliure = Convert.ToInt32(SimulationSettings.PaintableComputerFailureTime);

			communicationEnabled = true;

            // Stvaranje PCjeva na zidu
			logging.Enqueue(
				new LogEntry(String.Format(UserMessages.SimulatorInitialization, simulationSettings.MaximumComputerCount)));
			if (log.IsInfoEnabled)
				log.Info(String.Format(UserMessages.SimulatorInitialization, simulationSettings.MaximumComputerCount));

			SplashOnTheWall((uint)simulationSettings.MaximumComputerCount, paintableComputerType);

			logging.Enqueue(new LogEntry(String.Format(UserMessages.SimulatorInialized, info.computerCount)));
			if (log.IsInfoEnabled)
				log.Info(String.Format(UserMessages.SimulatorInialized, info.computerCount));

			// Traenje susjeda
			uint neighbourCount = CalculateNeighbours(info.Environment.wallSize);

			// Stvori simulator thread (po defaultu pauzirani)
			simulatorThread = new Thread(new ThreadStart(ThreadProc));
			simulatorThread.Start();

			// Sve je OK
			logging.Enqueue(new LogEntry(UserMessages.SimulatorReady));
			if (log.IsInfoEnabled)
				log.Info(UserMessages.SimulatorReady);


			//
			info.simulatorState = SimulatorStates.ReadyToRun;
		}

		/// <summary>
		/// 
		/// </summary>
		public override void Terminate()
		{
			if (simulatorThreadSynchronization != null)
			{
				logging.Enqueue(new LogEntry(UserMessages.SimulatorThreadEnding));
				if (log.IsInfoEnabled)
					log.Info(UserMessages.SimulatorThreadEnding);

				//Koristimo info varijablu za thread-safe rad
				lock (simulatorThreadSynchronization)
				{
					simulatorThreadSynchronization.DoFinish = true;
				}
				if (simulatorThread != null)
				{
					simulatorThread.Join();
					simulatorThread = null;
				}
                enviromentFeeders.Clear();

				logging.Enqueue(new LogEntry(UserMessages.SimulatorThreadEnded));
				if (log.IsInfoEnabled)
					log.Info(UserMessages.SimulatorThreadEnded);
			}
			GC.Collect();
		}

		/// <summary>
		/// Returns the simulator status
		/// </summary>
		/// <returns></returns>
		public override SimulatorInfo SimulatorStatus
		{
			get { return info; }
		}


		/// <summary>
		/// Gets the computer info from point.
		/// </summary>
		/// <param name="point">The point.</param>
		/// <returns></returns>
		public PConTheWall GetComputerInfoFromPoint(Point location)
		{
			PConTheWall pc;
			for (int i = 0; i < computerList.Count; i++)
			{
				pc = this.computerList[i];
				// "5" bi trebalo biti pointsize
				if (PointIsInRadius(pc.Location, location, 5))
					return pc;
			}

			return null;
		}


		/// <summary>
		/// Invades the computer.
		/// </summary>
		/// <param name="computerIndex">Index of the computer.</param>
		/// <param name="processFragmentType">Type of the process fragment.</param>
		/// <param name="state">The state.</param>
		public override void InvadeComputer(int computerIndex, ProcessFragmentBase processFragment)
		{
			// if none was chosen, insert into first computer
			if ((computerList.Count <= computerIndex) && (computerIndex < 0))
				computerIndex = 0;


			// construct a new packet containing the chosen ProcessFragment
			ProcessFragmentPacket packet =
				new ProcessFragmentPacket(processFragment, 0, computerList[computerIndex].PaintableComputer.ComputerId);

			computerList[computerIndex].PaintableComputer.TransmissionAntenna = packet;
			computerList[computerIndex].IsPortal = true;
            computerList[computerIndex].ProcesFragmentID = processFragment.ProcessFragmentId;

            logging.Enqueue(new LogEntry(String.Format(UserMessages.ComputerInvaded, computerList[computerIndex].ID,
                                                       computerList[computerIndex].Location.ToString(),
                                                       Plugins.ProcessFragmentIdTranslations[processFragment.ProcessFragmentId])));
		}


		/// <summary>
		/// Provides access to the simlation parameters
		/// </summary>
		/// <value>The simulation settings.</value>
		public override EngineSettings SimulationSettings
		{
			get { return simulationSettings; }
			set { simulationSettings = value; }
		}


		/// <summary>
		/// Disables the computer or removes it.
		/// </summary>
		/// <param name="index">The index.</param>
		/// <param name="remove">if set to <c>true</c> the compter will be removed.</param>
		private void DisableComputer(int index, bool remove)
		{
			lock (simulatorThreadSynchronization)
			{
				int removeAt = Convert.ToInt32(index);

				if ((removeAt >= 0) && (removeAt < info.computerCount))
				{
                    logging.Enqueue(new LogEntry(String.Format(UserMessages.SimulatorPaintableComputerDestructionEx,
                                                    computerList[removeAt].ID,
                                                    computerList[removeAt].Location.ToString())));

					if (log.IsInfoEnabled)
						log.Info(UserMessages.SimulatorPaintableComputerDestruction + computerList[removeAt].PaintableComputer.ComputerId.ToString());


					computerList[removeAt].IsAlive = false;
				}
			}
		}

		/// <summary>
		/// Disables the computers or removes it.
		/// </summary>
		/// <param name="computers">The computers.</param>
		/// <param name="remove">if set to <c>true</c> the compters will be removed.</param>
		public override void DisableComputers(IList<int> computers, bool remove)
		{
			List<int> temp = new List<int>(computers);

			temp.Sort();

			for (int i = temp.Count - 1; i >= 0; i--)
				DisableComputer(temp[i], remove);

			CalculateNeighbours(info.Environment.wallSize);
		}

		/// <summary>
		/// Check if poit is in polygon
		/// </summary>
		/// <param name="p"></param>
		/// <param name="poly"></param>
		/// <returns></returns>
		private bool PointInPoligon(Point p, Point[] poly)
		{
			// the crossing number counter
			int cn = 0;
			// loop through all edges of the polygon
			for (int i = 0; i < poly.Length; i++)
			{
				// edge from V[i] to V[i+1]
				if (((poly[i].Y <= p.Y) && (poly[(i + 1) % poly.Length].Y > p.Y)) // an upward crossing
					|| ((poly[i].Y > p.Y) && (poly[(i + 1) % poly.Length].Y <= p.Y)))
				{
					// a downward crossing
					// compute the actual edge-ray intersect x-coordinate
					float vt = (float)(p.Y - poly[i].Y) / (poly[(i + 1) % poly.Length].Y - poly[i].Y);
					if (p.X < poly[i].X + vt * (poly[(i + 1) % poly.Length].X - poly[i].X)) // P.x < intersect
						++cn; // a valid crossing of y=P.y right of P.x
				}
			}
			return (cn % 2 == 1); // 0 if even (out), and 1 if odd (in)
		}


        /// <summary>
        /// 
        /// </summary>
        public int GetLivePCCount
        {
            get
            {
                // Da donja petlja ne bi ostala zauvijek u petlji, moramo se pobrinuti
                // da count nije veci od broja zivih compjutera
                int liveCount = 0;
                for (int i = 0; i < computerList.Count; i++)
                    if (computerList[i].IsAlive) liveCount++;
                return liveCount;
            }
        }

		/// <summary>
		/// Removes a random computer.
		/// </summary>
		public void RemoveRandomComputers(int count)
		{
            // Da donja petlja ne bi ostala zauvijek u petlji, moramo se pobrinuti
            // da count nije veci od broja zivih compjutera
            int liveCount = GetLivePCCount;
            if (liveCount < count)
                count = liveCount;            

			int removeAt;
            while (count > 0)
            {
                int attempts = 0;
                do
                {
                    attempts++;
                    removeAt = RandomNumberGenerator.GetRandomNumberRange(0, computerList.Count - 1);
                } while ((!computerList[removeAt].IsAlive) && (attempts < 100));
                //Ako u 100 pokusaja ne pronadjemo pravog kandidata, probaj slijedno
                if (attempts >= 100)
                {
                    removeAt = -1;
                    for (int i = 0; i < computerList.Count; i++)
                        if (computerList[i].IsAlive)
                        {
                            removeAt = i;
                            break;
                        }
                    // Vise nema zivih PCja...
                    if (removeAt == -1)
                    {
                        break;
                    }
                }
                DisableComputer(removeAt, false);
                count--;
            }
            CalculateNeighbours(info.Environment.wallSize);
		}
        /// <summary>
        /// 
        /// </summary>
        /// <param name="Percent"></param>
        public void RemoveRandomComputers(double Percent)
        {
            int count = Convert.ToInt32((GetLivePCCount * Percent) / 100);
            RemoveRandomComputers(count);
        }
        
        /// <summary>
		/// Removes the computers in polygon.
		/// </summary>
		/// <param name="polygon">The polygon.</param>
		public void RemoveComputersInPolygon(Point[] polygon)
		{
			if (polygon == null)
				return;

			List<int> indices = new List<int>();

			// Idemo po svim PCjima i traimo one koji su u polygonu
			for (int i = 0; i < computerList.Count; i++)
				if (PointInPoligon(computerList[i].Location, polygon))
					indices.Add(i);

			DisableComputers(indices, false);
		}

		#region Experimental HP Colorizer
		
		/// <summary>
		/// Computers the colorizer.
		/// </summary>
		/// <param name="computer">The computer.</param>
		/// <returns></returns>
		private Color ComputerToColorConverter(PConTheWall computer)
		{
			return homepageColorizerDelegate(computer.PaintableComputer.LocalHomepage);
		}

		/// <summary>
		/// Colorizes the paintable computer homepages.
		/// </summary>
		/// <param name="colorizer">The colorizer.</param>
		/// <returns>
		/// A list of colors matching the list of paintable computers
		/// </returns>
		public override List<Color> ColorizeHomepages(FriendlyType colorizer)
		{
			List<Color> retval = null;

			if (colorizer == null)
				homepageColorizer = defaultColorizer;
			else
			{
				try
				{
					homepageColorizer = (HomepageColorizer)Activator.CreateInstance(colorizer);
				}
				catch
				{
					System.Diagnostics.Debug.Write("");
				}
			}

			// if none specified, use default
			if (homepageColorizer == null)
				homepageColorizer = defaultColorizer;

			homepageColorizerDelegate = new Converter<Homepage, Color>(homepageColorizer.Colorize);

			// perform a conversion of computers to color, which in turn calls the pfrag colorizter
			try
			{
				return computerList.ConvertAll<Color>(new Converter<PConTheWall, Color>(ComputerToColorConverter));
			}
			catch
			{
				homepageColorizerDelegate = new Converter<Homepage, Color>(defaultColorizer.Colorize);
				homepageColorizer = null;
				return computerList.ConvertAll<Color>(new Converter<PConTheWall, Color>(ComputerToColorConverter));
			}
		}

		#endregion

        #endregion

    }
}