/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using PaintableSimulator.Engine.PaintableComputers.Sensors;
using System.Drawing;

namespace PaintableSimulator.Engine.PaintableComputers.Sensors
{
	/// <summary>
	/// 
	/// </summary>
	public class AntWalkerDataFeeder : EnvironmentDataFeeder
	{
		private Point centerPoint;
		private int minimalRadius;
		public const double MAXIMUMPRESSURE = 100.0;

		/// <summary>
		/// Initializes a new instance of the <see cref="T:AntWalkerDataFeeder"/> class.
		/// </summary>
		/// <param name="wallSize">Size of the wall.</param>
		public AntWalkerDataFeeder(Size wallSize)
			: base(wallSize)
		{

			Random rnd = new Random();

			// center point
			centerPoint = new Point(wallSize.Width / 2, wallSize.Height / 2);

			minimalRadius = wallSize.Width;
			if (minimalRadius > wallSize.Height)
				minimalRadius = wallSize.Height;
		}

		/// <summary>
		/// Gets the environmental value.
		/// </summary>
		/// <param name="location">The location.</param>
		/// <param name="ticks">The ticks.</param>
		/// <returns></returns>
		public override double GetValue(System.Drawing.Point location, uint ticks)
		{
			double time = ticks / 100;
			double angle;
			double radius;

			// the ant moves on a helix path
			angle = (time % 360) * Math.PI / 180;
			radius = time / 4 % minimalRadius;


			double antLocationX = radius * Math.Cos(angle) + centerPoint.X;
			double antLocationY = radius * Math.Sin(angle) + centerPoint.Y;


			double sensorDistance = 0.1+Math.Sqrt(Math.Pow(antLocationX - location.X, 2) + Math.Pow(antLocationY - location.Y, 2)) ;
			return MAXIMUMPRESSURE / sensorDistance;
		}

	}
}
