/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace PaintableSimulator.Engine.PaintableComputers.Sensors
{
	/// <summary>
	/// 
	/// </summary>
	public class SimpleTemperatureDataFeeder : EnvironmentDataFeeder
	{

		private Point heatSource;
		private double itensity;

		/// <summary>
		/// Initializes a new instance of the <see cref="T:SimpleTemperatureDataFeeder"/> class.
		/// </summary>
		/// <param name="wallSize">Size of the wall.</param>
		public SimpleTemperatureDataFeeder(Size wallSize)
			: base(wallSize)
		{
			////////////////////////////////////////////////////////////////
			// HERE YOU CAN READ THE INITIAL DATA FROM FILES !!!!!!!!!!!!!!!
			////////////////////////////////////////////////////////////////

			Random rnd = new Random();
			// light source is at the center of the wall
			int x = rnd.Next(wallSize.Width);
			int y = rnd.Next(wallSize.Height);

			heatSource = new Point(x, y);
			itensity = 70.0;
		}

		/// <summary>
		/// Gets the environmental value.
		/// </summary>
		/// <param name="location">The location.</param>
		/// <param name="ticks">The ticks.</param>
		/// <returns></returns>
		public override double GetValue(System.Drawing.Point location, uint ticks)
		{

			////////////////////////////////////////////////////////////////
			// HERE YOU CAN'T DO ANY IO. OTHERWISE SecurityException WILL 
			// BE THROWN EVERY SIMULATOR CYCLE AND THE SIMULATION SPEED WILL 
			// BE GREATLY REDUCED
			////////////////////////////////////////////////////////////////


			double dx = (heatSource.X - location.X) / 2.0;
			double dy = (heatSource.Y - location.Y) / 2.0;

			double squareDistance = Math.Sqrt((dx * dx + dy * dy) / (wallSize.Width * wallSize.Width + wallSize.Height * wallSize.Height));

			// the light changes sinusioidally
			return itensity / squareDistance * (1 + Math.Cos(ticks / 4));

		}
	}
}
