/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using PaintableSimulator.Engine.PaintableComputers.OperatingSystem;
using PaintableSimulator.Engine;
using PaintableSimulator.Engine.PaintableComputers;

namespace PaintableSimulator.Programs
{
	#region Colorizer

	/// <summary>
	/// 
	/// </summary>
	public class AdaptiveShadowFaxColorizer : HomepageColorizer
	{
		// private static double maximumReachedDistance = 0.0;

		/// <summary>
		/// Colorizes the specified homepage.
		/// </summary>
		/// <param name="homepage">The homepage.</param>
		/// <returns></returns>
		public override Color Colorize(Homepage homepage)
		{
			Color retval = Color.WhiteSmoke;

			try
			{
				for (int i = 0; i < homepage.Count; i++)
				{
					if (homepage.KeyList[i] == 0x54AD0EFB)
					{
						if (homepage.DataList[i].Count >= 16)
						{
							retval = (Color)homepage.DataList[i][15];
						}
					}
				}
			}
			catch
			{
			}

			return retval;
		}
	}

	#endregion


	public enum ShadowZFaxPersonality
	{
		Initiator,
		Listener,
		Carrier
	} ;

	public enum ShadowZFaxInitiatorState
	{
		brandNew,
		enlightment,
		brightness,
		sending,
		waiting,
		killing,
		dead
	} ;

	public enum ShadowZFaxListenerState
	{
		brandNew,
		enlightment,
		brightness,
		waiting,
		answering,
		dieing
	} ;

	public enum ShadowZFaxCarrierState
	{
		setup,
		patchThrough,
		gone,
	} ;


	[FriendlyDescription("He would not tolerate a bridle or saddle, and only carried Gandalf by choice...")]
	[FriendlyName("Adaptive Shadowfax messenger")]
	[Serializable()]
	[DefaultHomepageColorizerAttribute(typeof(AdaptiveShadowFaxColorizer))]
	public sealed class ShadowZFax : ProcessFragmentBase
	{
		#region Process Fragment Identity

		public override uint ProcessFragmentId
		{
			get { return 0x54AD0EFB; }
		}

		#endregion

		#region ShadowZFax Local state

		private ShadowZFaxPersonality personality;

		[Browsable(true)]
		[Category("Personality of ShadowZFax")]
		public ShadowZFaxPersonality Personality
		{
			get { return personality; }
			set { personality = value; }
		}

		private string name = "Alice";

		[Browsable(true)]
		[Category("Name")]
		public string Name
		{
			get { return name; }
			set { name = value; }
		}

		private string destination = "Bob";

		[Browsable(true)]
		[Category("Sending to")]
		public string Destination
		{
			get { return destination; }
			set { destination = value; }
		}

		private int channel = 0;

		[Browsable(true)]
		[Category("Channel")]
		public int Channel
		{
			get { return channel; }
			set { channel = value; }
		}

		#endregion

		private static int TimeoutSearch = 10000;
		private static int RetriesCount = 50;
		private static int MessagesCount = 200;
		private static int HoldTime = 100;
		private static int TimeoutWait = 100000;
		private static int NumberOfChannels = 64;
		private static int ChannelOffset = 16;
        private static int TimeOut = 8;
        private static int TimeOutInitiator = 128;
        private static int TimeOutListener = -TimeOutInitiator/4;
        private static int TimeOutCarrier = -TimeOut - 1;


		/// <summary>
		/// This method is called when the PFrag first enters a Paintable.
		/// </summary>
		/// <param name="initialState">The initial state vector passed on by his ancestor.</param>
		/// <remarks>
		/// </remarks>
		public override void Install()
		{
            object[] timedPost = new object[2];
            timedPost[0] = 0;
            timedPost[1] = "";

			int c = Api.GetPostCount(this.ProcessFragmentId);
			if (c > 0 && this.Personality != ShadowZFaxPersonality.Initiator && this.Personality != ShadowZFaxPersonality.Listener)
			{
				Api.QueueUninstall();
				return;
			}

			if (this.Personality == ShadowZFaxPersonality.Initiator)
			{
				Api.PostEntry(this.ProcessFragmentId, 0, "ShadowZFax:Initiator;");
				Api.PostEntry(this.ProcessFragmentId, 0, "ShadowZFax:Initiator;");
				Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:BrandNew;");
				Api.PostEntry(this.ProcessFragmentId, 2, this.Name);
				Api.PostEntry(this.ProcessFragmentId, 3, Api.GetComputerId());
				Api.PostEntry(this.ProcessFragmentId, 4, this.Destination);
				Api.PostEntry(this.ProcessFragmentId, 7, (int)0);

				// other posts - just to fill
				Api.PostEntry(this.ProcessFragmentId, 5, channel);
				Api.PostEntry(this.ProcessFragmentId, 6, "");
				Api.PostEntry(this.ProcessFragmentId, 8, "");
				Api.PostEntry(this.ProcessFragmentId, 9, 0);
				Api.PostEntry(this.ProcessFragmentId, 10, "");
				Api.PostEntry(this.ProcessFragmentId, 11, "");

				// Api.SetColorTrap(Color.Orange);
				Api.PostEntry(this.ProcessFragmentId, 15, Color.Orange);

				// isprazni sve kanale
				for (int i = 0; i < NumberOfChannels; i++)
					Api.PostEntry(this.ProcessFragmentId, i + ChannelOffset, timedPost);
			}
			else if (this.Personality == ShadowZFaxPersonality.Listener)
			{
				Api.PostEntry(this.ProcessFragmentId, 0, "ShadowZFax:Listener;");
				Api.PostEntry(this.ProcessFragmentId, 0, "ShadowZFax:Listener;");
				Api.PostEntry(this.ProcessFragmentId, 1, "Listener:BrandNew;");
				Api.PostEntry(this.ProcessFragmentId, 2, this.Name);
				Api.PostEntry(this.ProcessFragmentId, 3, Api.GetComputerId());
				//Api.PostEntry(this.ProcessFragmentId, 4, this.Name);

				// other posts - just to fill
				Api.PostEntry(this.ProcessFragmentId, 4, "");
				Api.PostEntry(this.ProcessFragmentId, 5, channel);
				Api.PostEntry(this.ProcessFragmentId, 6, (int)TimeoutWait);
				Api.PostEntry(this.ProcessFragmentId, 7, "");
				Api.PostEntry(this.ProcessFragmentId, 8, "");
				Api.PostEntry(this.ProcessFragmentId, 9, "");
				Api.PostEntry(this.ProcessFragmentId, 10, "");
				Api.PostEntry(this.ProcessFragmentId, 11, "");

				// Api.SetColorTrap(Color.Brown);
				Api.PostEntry(this.ProcessFragmentId, 15, Color.Brown);

				// isprazni sve kanale
				for (int i = 0; i < NumberOfChannels; i++)
					Api.PostEntry(this.ProcessFragmentId, i + ChannelOffset, timedPost);
			}
			else
			{
				Api.PostEntry(this.ProcessFragmentId, 0, "ShadowZFax:Carrier");
				Api.PostEntry(this.ProcessFragmentId, 0, "ShadowZFax:Carrier");
				Api.PostEntry(this.ProcessFragmentId, 1, "Carrier:Setup;");
				Api.PostEntry(this.ProcessFragmentId, 2, this.Name);
				Api.PostEntry(this.ProcessFragmentId, 3, Api.GetComputerId());
				Api.PostEntry(this.ProcessFragmentId, 5, channel);

				// other posts - just to fill
				Api.PostEntry(this.ProcessFragmentId, 4, "");
				Api.PostEntry(this.ProcessFragmentId, 6, (int)TimeoutWait);
				Api.PostEntry(this.ProcessFragmentId, 7, "");
				Api.PostEntry(this.ProcessFragmentId, 8, "");
				Api.PostEntry(this.ProcessFragmentId, 9, "");
				Api.PostEntry(this.ProcessFragmentId, 10, "");
				Api.PostEntry(this.ProcessFragmentId, 11, "");

				//Api.SetColorTrap(Color.Gray);
				Api.PostEntry(this.ProcessFragmentId, 15, Color.Gray);


				// isprazni sve kanale
				for (int i = 0; i < NumberOfChannels; i++)
					Api.PostEntry(this.ProcessFragmentId, i + ChannelOffset, timedPost);
			}
		}

		/// <summary>
		/// This method is called after the PFrag queued himself for uninstall and the OS approves an uninstall.
		/// </summary>
		/// <param name="remainingTimeslots"></param>
		/// <remarks>
		/// Here you sould do some pre uninstall processing.
		/// Removing posts from the homepage is unnecessary because the os does that for you automatically.
		/// </remarks>
		public override void Uninstall(byte remainingTimeslots)
		{
		}

		/// <summary>
		/// This method is called if the PFrag requested a transfer to the neighboring particles and
		/// his request was approved.
		/// </summary>
		/// <param name="copy">The copy of the ProcessFragmentBase.</param>
		public override void TransferGranted(ref ProcessFragmentBase copy)
		{
			ShadowZFax ss = (ShadowZFax)copy;
			ss.Personality = ShadowZFaxPersonality.Carrier;
			ss.Name = "";
			ss.Destination = "";
		}

		/// <summary>
		/// This method is called if the PFragment requested a transfer to the neighboring particles and
		/// his request was denied.
		/// </summary>
		public override void TransferRefused()
		{
		}

		/// <summary>
		/// The main processing method for Process Fragments
		/// </summary>
		/// <param name="remainingTimeslots">A value indicating the available processing time for the Update method.</param>
		/// <remarks>
		/// The time slot of 0 indicates less time available for processing and 255 more time.
		/// This is just a suggested value, it has no bearing on the execution of the Update method (yet).
		/// </remarks>
		public override void Update(byte remainingTimeslots)
		{
			ShadowZFaxPersonality personality = getPersonality();

			string post = (string)Api.GetPost(this.ProcessFragmentId, 0);

			if (personality == ShadowZFaxPersonality.Initiator)
			{
				ShadowZFaxInitiatorState initiator = getInitiatorState();

				if (initiator == ShadowZFaxInitiatorState.brandNew)
				{
					// skroz novo - zapocni trazenje staze
					// ili mozda ponovljen trazenje staze

					// pokrenuti slanje carriera dalje, ako ga nema, naravno
					int mirrorsCount = Api.GetIOSpaceCount();

					// ako trenutno nema mirrora, salji kasnije
					if (mirrorsCount == 0)
						return;

					for (int i = 0; i < mirrorsCount; i++)
					{
						string type = (string)Api.GetIOPost(i, this.ProcessFragmentId, 2);
						if (type == null || (!type.StartsWith("Carrier") && !type.StartsWith("Listener")))
						{
							Api.QueueTransfer(i);
						}
					}

					// slozi poruku:
					// ENLIGHT = destination; FROM = source; ROAD = thisID;
					string destination = (string)Api.GetPost(this.ProcessFragmentId, 4);
					string source = (string)Api.GetPost(this.ProcessFragmentId, 2);
					string s = "ENLIGHT=" + destination + ";" + "FROM=" + source + ";" + "ROAD=" + Api.GetComputerId().ToString() + ";";
					int ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
                    object[] timedPost = new object[2];
                    timedPost[0] = TimeOut;
                    timedPost[1] = s;
                    Api.PostEntry(this.ProcessFragmentId, ch, timedPost);

					// oznaci cekanje za TimeOutInitiator i predji u stanje cekanja povratne poruke
					Api.PostEntry(this.ProcessFragmentId, 6, (int)0);
					Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:Enlightment");
					//Api.SetColorTrap(Color.Yellow);
					Api.PostEntry(this.ProcessFragmentId, 15, Color.Yellow);
					return;
				}
				else if (initiator == ShadowZFaxInitiatorState.enlightment)
				{
                    // trik
                    int chx = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
                    object[] timedPost = (object[])Api.GetPost(this.ProcessFragmentId, chx);
                    timedPost[0] = ((int)timedPost[0]) - 1;
                    if ((int)timedPost[0] < 0)
                    {
                        timedPost[1] = "";
                    }
                    Api.PostEntry(this.ProcessFragmentId, chx, timedPost);

                    if ((int)timedPost[0] < -TimeOutInitiator)
                    {
                        Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:BrandNew");
                    }

					// povecaj vrijeme cekanja
					// ako je proslo vrijeme za TimeOutInitiator, ponovi zahtjev za dobivanjem staze
					int clock = (int)Api.GetPost(this.ProcessFragmentId, 6);
					clock++;
					Api.PostEntry(this.ProcessFragmentId, 6, clock);
					if (clock > TimeOutInitiator)
					{
						// ako je predjen TimeOutInitiator, povecaj broj pokusaja, 
						// a ako je broj pokusaja presao predvidjen broj, napravi uninstall
						int retries = (int)Api.GetPost(this.ProcessFragmentId, 7);
						retries++;
						Api.PostEntry(this.ProcessFragmentId, 7, retries);
						if (retries > RetriesCount)
						{
							Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:Dead");
							return;
							//Api.QueueUninstall();
						}
						Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:BrandNew");
						return;
					}

					bool foundPath = false;
					string shortestPath = "";
					for (int i = 0; i < Api.GetIOSpaceCount(); i++)
					{
						int ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
                        object [] timedPost2 = (object [])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                        string message = (string)timedPost2[1];
						if (message != null && message.StartsWith("RESPENLIGHT"))
						{
							// dosao odgovor - trazeno
							foundPath = true;
							// shortestPath = GetShortestPath(shortestPath, message);
							shortestPath = ReversePath(ExtractRoad(message));
						}
					}

					if (foundPath)
					{
						// ako je staza pronadjena, predji u sljedece stanje
						// iz kojeg ce se poslati poruka
						// shortestPath = ReversePath(ExtractPath(shortestPath));
						Api.PostEntry(this.ProcessFragmentId, 6, (int)0);
						Api.PostEntry(this.ProcessFragmentId, 7, (int)0);
						Api.PostEntry(this.ProcessFragmentId, 8, shortestPath);
						Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:Brightness");
						//Api.SetColorTrap(Color.Green);
						Api.PostEntry(this.ProcessFragmentId, 15, Color.Green);

					}
					else
					{
						//Api.SetColorTrap(Color.YellowGreen);
						Api.PostEntry(this.ProcessFragmentId, 15, Color.YellowGreen);

					}
				}
				else if (initiator == ShadowZFaxInitiatorState.brightness)
				{
					// staza je pronadjenja, posalji poruku
					int mesgCount = (int)Api.GetPost(this.ProcessFragmentId, 9);
                    if(mesgCount <= 1)
    					mesgCount++;
					Api.PostEntry(this.ProcessFragmentId, 9, mesgCount);
					if (mesgCount < MessagesCount)
					{
						Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:Sending");
						//Api.SetColorTrap(Color.Green);
						Api.PostEntry(this.ProcessFragmentId, 15, Color.Green);
					}
					else
					{
						Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:Killing");
						//Api.SetColorTrap(Color.Pink);
						Api.PostEntry(this.ProcessFragmentId, 15, Color.Pink);
					}
				}
				else if (initiator == ShadowZFaxInitiatorState.sending)
				{
					string path = (string)Api.GetPost(this.ProcessFragmentId, 8);
					int mesgCount = (int)Api.GetPost(this.ProcessFragmentId, 9);
					string message = "MESG=" + mesgCount.ToString() + ";" + "ROAD=" +
									 path + ";" + "PATH=" + RemoveThisFromPath(ReversePath(path) + ";");

					int ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);

                    object[] timedPost = new object[2];
                    timedPost[0] = TimeOutInitiator;
                    timedPost[1] = message;
                    Api.PostEntry(this.ProcessFragmentId, ch, timedPost);

					Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:Waiting");
					//Api.SetColorTrap(Color.Lime);
					Api.PostEntry(this.ProcessFragmentId, 15, Color.Lime);
				}
				else if (initiator == ShadowZFaxInitiatorState.waiting)
				{
					// poruka poslana, ocekuje se odgovor
					// povecaj vrijeme cekanja
					// ako je proslo vrijeme za TimeOutInitiator, ponovi slanje poruke
					int clock = (int)Api.GetPost(this.ProcessFragmentId, 6);
					clock++;
					Api.PostEntry(this.ProcessFragmentId, 6, clock);
					if (clock > TimeOutInitiator)
					{
						// ako je predjen TimeOutInitiator, povecaj broj pokusaja, 
						// a ako je broj pokusaja presao predvidjen broj, napravi uninstall
                        Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:BrandNew");
						initiator = ShadowZFaxInitiatorState.brightness;
						int retries = (int)Api.GetPost(this.ProcessFragmentId, 7);
						retries++;
						Api.PostEntry(this.ProcessFragmentId, 7, retries);
						if (retries > RetriesCount)
						{
							Api.QueueUninstall();
						}

						return;
					}


					// dobio poruku, slozi odgovor i predji u stanje cekanja :)
					int ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
					
                    object[] timedPost = (object[])Api.GetPost(this.ProcessFragmentId, ch);
                    
                    // moded
                    timedPost[0] = ((int)timedPost[0]) - 1;
                    if ((int)timedPost[0] < 0)
                    {
                        timedPost[1] = "";
                    }
                    Api.PostEntry(this.ProcessFragmentId, ch, timedPost);

                    string mesg = (string)timedPost[1];
					

                    int oldMesgIndex = 0;
                    if (mesg.StartsWith("MESG"))
                    {
                        int index = mesg.Substring(5).IndexOf(";");
                        oldMesgIndex = Convert.ToInt32(mesg.Substring(5).Substring(0, index));
                    }
                    else
                    {
                        oldMesgIndex = (int)Api.GetPost(this.ProcessFragmentId, 9);
                    }

					// procitaj odgovor i napisi novu poruku ako compID i broj odgovaraju...

					for (int i = 0; i < Api.GetIOSpaceCount(); i++)
					{
						ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
						object[] timedPost2 = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                        string ioMesg = (string)timedPost2[1];
						if (ioMesg != null && ioMesg != "")
						{
							if (ioMesg.StartsWith("RESPMESG"))
							{
								int newMesgIndex = 0;
								int index = ioMesg.Substring(9).IndexOf(";");
								newMesgIndex = Convert.ToInt32(ioMesg.Substring(9).Substring(0, index));

								string pathX = ExtractPath(ioMesg);
								if (newMesgIndex == oldMesgIndex && getLastIDPathRoad(pathX) == Api.GetComputerId())
								{
									newMesgIndex++; // nova (sljedeca) poruka
									//string path = ExtractRoad(ioMesg);
									//string message = "MESG=" + newMesgIndex.ToString() + ";" + "ROAD=" +
									//             ReversePath(path) + ";" + "PATH=" + RemoveThisFromPath(path + ";");


									Api.PostEntry(this.ProcessFragmentId, 9, newMesgIndex);
									Api.PostEntry(this.ProcessFragmentId, 15, Color.BlueViolet);
									//Api.SetColorTrap(Color.BlueViolet);
									Api.PostEntry(this.ProcessFragmentId, 7, "BlueViolet");
									// int ch = channelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
									// Api.PostEntry(this.ProcessFragmentId, ch, message);
									Api.PostEntry(this.ProcessFragmentId, 1, "Initiator:Sending");
								}
							}
						}
					}


					// posalji poruku za prekid veze 

					// odgovor dosao, salji dalje ili prekini vezu
				}
				else if (initiator == ShadowZFaxInitiatorState.killing)
				{
					// ceka se potvrda za prekidanje veze, pa se ubija kasnije
				}
				else
				{
					// ready to die ;)
					Api.QueueUninstall();
					return;
				}
			}
			else if (personality == ShadowZFaxPersonality.Listener)
			{
				ShadowZFaxListenerState listener = getListenerState();

				if (listener == ShadowZFaxListenerState.brandNew)
				{
					// brand new, ocekujem trazenje staze
					int clock = (int)Api.GetPost(this.ProcessFragmentId, 6);
					clock--;
					Api.PostEntry(this.ProcessFragmentId, 6, clock);

					if (clock < 0)
					{
						Api.QueueUninstall();
					}
					else
					{
						for (int iterator = 0; iterator < NumberOfChannels; iterator++)
						{
							int ch = iterator + ChannelOffset;
							int mirrorsCount = Api.GetIOSpaceCount();
							for (int i = 0; i < mirrorsCount; i++)
							{
                                object[] timedPost = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                                string mesg = (string)timedPost[1];
								if (mesg != null && mesg != "")
								{
									string myName = (string)Api.GetPost(this.ProcessFragmentId, 2);
									string expectedMesg = "ENLIGHT=" + myName;
									if (mesg.StartsWith(expectedMesg))
									{
										// DOSLO JE NESTO, spremi broj kanala
										Api.PostEntry(this.ProcessFragmentId, 5, ch - ChannelOffset);

										// goto waiting
										Api.PostEntry(this.ProcessFragmentId, 14, TimeOut);
										Api.PostEntry(this.ProcessFragmentId, 1, "Listener:Enlightment");
									}
								}
							}
						}
					}
				}
				else if (listener == ShadowZFaxListenerState.enlightment)
				{
					// dobio prvu stazu, vracam natrag punu stazu
					// brand new, ocekujem trazenje staze
					int clockPath = (int)Api.GetPost(this.ProcessFragmentId, 14);
					clockPath--;
					Api.PostEntry(this.ProcessFragmentId, 14, clockPath);
					int ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);

					if (clockPath < 0)
					{
						string myName = (string)Api.GetPost(this.ProcessFragmentId, 2);
						// predji u stanje cekanja
						Api.PostEntry(this.ProcessFragmentId, 1, "Listener:Waiting");

						string mesg = (string)Api.GetPost(this.ProcessFragmentId, 4);
						// iscitavamo stazu
						int indexRoad = mesg.IndexOf("ROAD=");
						string road = mesg.Substring(indexRoad + 5);

						// gradimo novu stazu tako da na kraj prve dodamo sebe
						// i onda je okrenemo (priprema za slanje)
						string newRoad = road.Substring(0, road.Length - 1);

						string newPath = newRoad;
						newRoad += "," + Api.GetComputerId().ToString();
						newRoad = ReversePath(newRoad);

						// nova staza nam je jednaka staroj preokrenutoj
						// newPath = ReversePath(newPath);

						// slozimo poruku i posaljemo je:
						string newMesg = "RESPENLIGHT=" + myName + ";" +
										 "ROAD=" + newRoad + ";PATH=" + newPath + ";";

						ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
                        object[] timedPost = new object[2];
                        timedPost[0] = TimeOut;
                        timedPost[1] = newMesg;
						Api.PostEntry(this.ProcessFragmentId, ch, timedPost);
					}
					else
					{
						int mirrorsCount = Api.GetIOSpaceCount();
						for (int i = 0; i < mirrorsCount; i++)
						{
                            object[] timedPost = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                            string mesg = (string)timedPost[1];
							if (mesg != null && mesg != "")
							{
								string myName = (string)Api.GetPost(this.ProcessFragmentId, 2);
								string expectedMesg = "ENLIGHT=" + myName;
								// pronadjena staza, posalji odgovor, predji u novo stanje
								if (mesg.StartsWith(expectedMesg))
								{
									int indexFrom = mesg.IndexOf("FROM=");
									/*
									// izbacimo sve prije FROM= i FROM=
									string temp = mesg.Substring(indexFrom + 5);
									int indexEndFrom = temp.IndexOf(";");

									// izbacimo sve poslije FROM=****;
									temp = temp.Substring(0, indexEndFrom);
									*/

									string oldMesg = (string)Api.GetPost(this.ProcessFragmentId, 4);

                                    string shorter = GetShortestPath(oldMesg, mesg);
									if (mesg.Contains(shorter))
										Api.PostEntry(this.ProcessFragmentId, 4, mesg);
								}
							}
						}
					}
				}
				else if (listener == ShadowZFaxListenerState.brightness)
				{
					// cekam prvu poruku i saljem odgovor
				}
				else if (listener == ShadowZFaxListenerState.waiting)
				{
					// dobio poruku, slozi odgovor i predji u stanje cekanja :)
					int ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
                    object[] timedPost = (object[])Api.GetPost(this.ProcessFragmentId, ch);
                    
                    // moded
                    timedPost[0] = ((int)timedPost[0]) - 1;
                    if ((int)timedPost[0] < 0)
                    {
                        timedPost[1] = "";
                    }
                    Api.PostEntry(this.ProcessFragmentId, ch, timedPost);


                    string mesg = (string)timedPost[1];
					int oldMesgIndex = 0;
					if (mesg.StartsWith("RESPMESG"))
					{
						int index = mesg.Substring(9).IndexOf(";");
						oldMesgIndex = Convert.ToInt32(mesg.Substring(9).Substring(0, index));
					}


					// procitaj novu poruku i napisi odgovor ako compID i broj odgovaraju...
					for (int i = 0; i < Api.GetIOSpaceCount(); i++)
					{
						ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
                        object[] timedPost2 = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                        string ioMesg = (string)timedPost2[1];
						if (ioMesg != null && ioMesg != "")
						{
                            if (ioMesg.StartsWith("ENLIGHT") && (int)timedPost[0] < TimeOutListener)
                            {
                                Api.PostEntry(this.ProcessFragmentId, 1, "Listener:Enlightment");
                                
                                // clock trick
                                Api.PostEntry(this.ProcessFragmentId, 14, -1);

                                
                                // string oldMesg = (string)Api.GetPost(this.ProcessFragmentId, 4);
                                string shorter = GetShortestPath("", ioMesg);
                                Api.PostEntry(this.ProcessFragmentId, 4, ioMesg);

                            }

							if (ioMesg.StartsWith("MESG"))
							{
								uint newMesgIndex = 0;
								int index = ioMesg.Substring(5).IndexOf(";");
								newMesgIndex = Convert.ToUInt32(ioMesg.Substring(5).Substring(0, index));

								string pathX = ExtractPath(ioMesg);
								if (newMesgIndex > oldMesgIndex && getLastIDPathRoad(pathX) == Api.GetComputerId())
								{
									string path = ExtractRoad(ioMesg);
									string message = "RESPMESG=" + newMesgIndex.ToString() + ";" + "ROAD=" +
												 ReversePath(path) + ";" + "PATH=" + RemoveThisFromPath(path + ";");

									Api.PostEntry(this.ProcessFragmentId, 15, Color.BlueViolet);
									//Api.SetColorTrap(Color.BlueViolet);
									Api.PostEntry(this.ProcessFragmentId, 7, "BlueViolet");

									ch = ChannelOffset + (int)Api.GetPost(this.ProcessFragmentId, 5);
                                    object[] timedPost3 = new object[2];
                                    timedPost3[0] = TimeOut;
                                    timedPost3[1] = message;
									Api.PostEntry(this.ProcessFragmentId, ch, timedPost3);
								}
							}
						}
					}

				}
				else if (listener == ShadowZFaxListenerState.answering)
				{
					// cekam sljedecu poruku / prekid veze i saljem odgovarajuci odgovor
				}
				else
				{
					// ubij se :)
				}

				// if I am listener
			}
			else
			{
				ShadowZFaxCarrierState carrier = getCarrierState();

				if (carrier == ShadowZFaxCarrierState.setup)
				{
					// prosiri se dalje
					for (int i = 0; i < Api.GetIOSpaceCount(); i++)
					{
						string message = (string)Api.GetIOPost(i, this.ProcessFragmentId, 0);
						if (message == null || !message.StartsWith("ShadowZFax"))
						{
							Api.QueueTransfer(i);
						}
					}

					// uspostavi red :), posalji dalje etc.

					Api.PostEntry(this.ProcessFragmentId, 6, (int)HoldTime);
					Api.PostEntry(this.ProcessFragmentId, 1, "Carrier:PatchThrough");

					for (int i = 0; i < NumberOfChannels; i++)
					{
						int ch = ChannelOffset + i;
                        object[] timedPost = new object[2];
                        timedPost[0] = TimeOut;
                        timedPost[1] = "";
						Api.PostEntry(this.ProcessFragmentId, ch, timedPost);
					}

					Api.PostEntry(this.ProcessFragmentId, 7, "lightpink");

					Api.PostEntry(this.ProcessFragmentId, 15, Color.LightPink);
				}
				else if (carrier == ShadowZFaxCarrierState.patchThrough)
				{
					// prosiri se dalje
					for (int i = 0; i < Api.GetIOSpaceCount(); i++)
					{
						string message = (string)Api.GetIOPost(i, this.ProcessFragmentId, 0);
						if (message == null || !message.StartsWith("ShadowZFax"))
						{
							Api.QueueTransfer(i);
						}
					}

					bool coloured = false;
					for (int iterator = 0; iterator < NumberOfChannels; iterator++)
					{
						int ch = ChannelOffset + iterator;
                        object[] timedPost = (object[])Api.GetPost(this.ProcessFragmentId, ch);
                        
                        // ADDED
                        timedPost[0] = ((int)timedPost[0])-1;
                        if ((int)timedPost[0] < 0)
                        {
                            timedPost[1] = "";
                        }

                        Api.PostEntry(this.ProcessFragmentId, ch, timedPost);


                        string mesg = (string)timedPost[1];
                        
						if (mesg != null)
						{
                            int xoldPriority = 0;
                            int xnewPriority = 0;
                            if (mesg == "")
                            {
                                xoldPriority = 0;
                            }
                            else if (mesg.StartsWith("ENLIGHT"))
                            {
                                xoldPriority = 1;
                            }
                            else if (mesg.StartsWith("RESPENLIGHT"))
                            {
                                xoldPriority = 2;
                            }
                            else if (mesg.StartsWith("MESG") || mesg.StartsWith("RESPMESG"))
                            {
                                xoldPriority = 3;
                            }


                            for (int i = 0; i < Api.GetIOSpaceCount(); i++)
                            {
                                object[] timedPost2 = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                                string ioMesg = (string)timedPost2[1];

                                if (ioMesg == null)
                                {
                                    xnewPriority = 0;
                                }
                                else
                                {
                                    if (ioMesg == "" && xnewPriority <= 0)
                                    {
                                        xnewPriority = 0;
                                    }
                                    else if (ioMesg.StartsWith("ENLIGHT") && xnewPriority <= 1)
                                    {
                                        xnewPriority = 1;
                                    }
                                    else if (ioMesg.StartsWith("RESPENLIGHT") && xnewPriority <= 2)
                                    {
                                        xnewPriority = 2;
                                    }
                                    else if (ioMesg.StartsWith("MESG") || mesg.StartsWith("RESPMESG") && xnewPriority <= 3)
                                    {
                                        xnewPriority = 3;
                                    }
                                }
                                    
                            }

                            try
							{
                                // NOVA PORUKA JE TU, PROPUSTI JE AKO TREBA :)
                                if (mesg.StartsWith("RESPENLIGHT") || mesg.StartsWith("RESPMESG") || mesg.StartsWith("MESG") || xnewPriority > 2)
                                {
                                    // odredimo "jacinu" stare poruke
                                    uint oldPriority = 0;
                                    if (mesg.StartsWith("MESG"))
                                    {
                                        int index = mesg.Substring(5).IndexOf(";");
                                        int num = Convert.ToInt32(mesg.Substring(5).Substring(0, index));
                                        oldPriority = (uint)(4 * num + 2);
                                    }
                                    else if (mesg.StartsWith("RESPMESG"))
                                    {
                                        int index = mesg.Substring(9).IndexOf(";");
                                        int num = Convert.ToInt32(mesg.Substring(9).Substring(0, index));
                                        oldPriority = (uint)(4 * num + 3);
                                    }

                                    // procitaj novu poruku i proslijedi je ako joj prioritet i compID odgovaraju...
                                    for (int i = 0; i < Api.GetIOSpaceCount(); i++)
                                    {
                                        object[] timedPost2 = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                                        string ioMesg = (string)timedPost2[1];
                                        if (ioMesg != null && ioMesg != "")
                                        {
                                            if (ioMesg.StartsWith("RESPMESG") || ioMesg.StartsWith("MESG"))
                                            {
                                                uint newPriority = 0;
                                                int boja = 0;
                                                if (ioMesg.StartsWith("MESG"))
                                                {
                                                    int index = ioMesg.Substring(5).IndexOf(";");
                                                    int num = Convert.ToInt32(ioMesg.Substring(5).Substring(0, index));
                                                    boja = num;
                                                    newPriority = (uint)(4 * num + 2);
                                                }
                                                else if (ioMesg.StartsWith("RESPMESG"))
                                                {
                                                    int index = ioMesg.Substring(9).IndexOf(";");
                                                    int num = Convert.ToInt32(ioMesg.Substring(9).Substring(0, index));
                                                    boja = num;
                                                    newPriority = (uint)(4 * num + 3);
                                                }

                                                string path = ExtractPath(ioMesg);

                                                int count = PathRoadCount(path);
                                                if (newPriority > oldPriority && getLastIDPathRoad(path) == Api.GetComputerId())
                                                {
                                                    // prenesi poruku preko na sebe
                                                    // string ioMesg = (string)Api.GetIOPost(bestIndex, this.ProcessFragmentId, ch);

                                                    path = "PATH=" + ExtractPath(ioMesg) + ";";
                                                    // string newPath = path + "," + Api.GetComputerId();
                                                    string newPath = path.Replace("," + Api.GetComputerId(), "");
                                                    ioMesg = ioMesg.Replace(path, newPath);

                                                    // Api.SetColorTrap(Color.BlueViolet);
                                                    if (boja > 127)
                                                        boja = 127;
                                                    boja += 128;

                                                    if (ioMesg.StartsWith("MESG"))
                                                    {
                                                        Api.PostEntry(this.ProcessFragmentId, 15, Color.FromArgb(boja, 32, 32));
                                                        coloured = true;
                                                    }
                                                    else
                                                    {
                                                        Api.PostEntry(this.ProcessFragmentId, 15, Color.FromArgb(32, boja, 32));
                                                        coloured = true;
                                                    }
                                                    Api.PostEntry(this.ProcessFragmentId, 7, "mesg_color");

                                                    object[] timedPost3 = new object[2];
                                                    timedPost3[0] = TimeOut;
                                                    timedPost3[1] = ioMesg;
                                                    Api.PostEntry(this.ProcessFragmentId, ch, timedPost3);
                                                }
                                            }
                                        }
                                    }


                                }
                                else if (mesg.StartsWith("ENLIGHT") || mesg.StartsWith("RESPENLIGHT") || xnewPriority > 1)
                                {
                                    int bestIndex = -1, bestCount = -1;

                                    if (mesg.StartsWith("RESPENLIGHT"))
                                    {
                                        string oldRoad = ExtractRoad(mesg);
                                        bestCount = PathRoadCount(oldRoad) - 1;
                                    }

                                    for (int i = 0; i < Api.GetIOSpaceCount(); i++)
                                    {
                                        object[] timedPost2 = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                                        string ioMesg = (string)timedPost2[1];
                                        if (ioMesg != null && ioMesg != "")
                                        {
                                            if (ioMesg.StartsWith("RESPENLIGHT"))
                                            /* || ioMesg.StartsWith("MESG") ||
                                        ioMesg.StartsWith("RESPMESG")*/
                                            {
                                                string path = ExtractPath(ioMesg);

                                                int count = PathRoadCount(path);
                                                if ((bestCount == -1 || bestCount > count) && getLastIDPathRoad(path) == Api.GetComputerId())
                                                    bestIndex = i;
                                            }
                                        }
                                    }

                                    if (bestIndex != -1)
                                    {
                                        object[] timedPost2 = (object[])Api.GetIOPost(bestIndex, this.ProcessFragmentId, ch);
                                        string ioMesg = (string)timedPost2[1];

                                        string path = "PATH=" + ExtractPath(ioMesg) + ";";
                                        // string newPath = path + "," + Api.GetComputerId();
                                        string newPath = path.Replace("," + Api.GetComputerId(), "");
                                        ioMesg = ioMesg.Replace(path, newPath);

                                        Api.PostEntry(this.ProcessFragmentId, 15, Color.BlueViolet);
                                        coloured = true;
                                        Api.PostEntry(this.ProcessFragmentId, 7, "violet");

                                        object[] timedPost3 = new object[2];
                                        timedPost3[0] = TimeOut;
                                        timedPost3[1] = ioMesg;
                                        Api.PostEntry(this.ProcessFragmentId, ch, timedPost3);
                                    }
                                }
                                else if (mesg == "" || mesg.StartsWith("ENLIGHT") || xnewPriority > 0)
								{
									int bestIndex = -1, bestCount = -1;
									if (mesg.StartsWith("ENLIGHT"))
									{
										string oldRoad = ExtractRoad(mesg);
										bestCount = PathRoadCount(oldRoad) - 1;
									}

									for (int i = 0; i < Api.GetIOSpaceCount(); i++)
									{
                                        object[] timedPost2 = (object[])Api.GetIOPost(i, this.ProcessFragmentId, ch);
                                        string ioMesg = (string)timedPost2[1];
										if (ioMesg != null && ioMesg != "")
										{
											if (ioMesg.StartsWith("ENLIGHT"))
											{
												string road = ExtractRoad(ioMesg);
												int count = PathRoadCount(road);
                                                if (!road.Contains(Api.GetComputerId().ToString()))
                                                {
                                                    bestIndex = i;
                                                    break;
                                                }
											}
										}
									}

									if (bestIndex != -1 /*|| mesg == ""*/ && !mesg.StartsWith("ENLIGHT"))
									{
                                        object[] timedPost2 = (object[])Api.GetIOPost(bestIndex, this.ProcessFragmentId, ch);
                                        string ioMesg = (string)timedPost2[1];
										string road = ExtractRoad(ioMesg);
										string newRoad = road + "," + Api.GetComputerId();
										ioMesg = ioMesg.Replace(road, newRoad);
										Api.PostEntry(this.ProcessFragmentId, 7, "yellowgreen");

										if (!coloured)
											Api.PostEntry(this.ProcessFragmentId, 15, Color.Gray);

                                        object[] timedPost3 = new object[2];
                                        timedPost3[0] = TimeOut;
                                        timedPost3[1] = ioMesg;
                                        if((int)timedPost[0] < TimeOutCarrier) // trik
    										Api.PostEntry(this.ProcessFragmentId, ch, timedPost3);
									}
								}

							}
							catch
							{
								
							}

						}
					}
					// poruka za prijenos detektirana na mirroru, stavi je na sebe
				}
				else if (carrier == ShadowZFaxCarrierState.gone)
				{
					// prosiri se dalje
					for (int i = 0; i < Api.GetIOSpaceCount(); i++)
					{
						string message = (string)Api.GetIOPost(i, this.ProcessFragmentId, 0);
						if (message == null || !message.StartsWith("ShadowZFax"))
						{
							Api.QueueTransfer(i);
						}
					}

					int htime = (int)Api.GetPost(this.ProcessFragmentId, 6);
					htime--;
					Api.PostEntry(this.ProcessFragmentId, 6, htime);
					if (htime > 0)
						return;

					Api.PostEntry(this.ProcessFragmentId, 1, "Carrier:Setup");
					//Api.SetColorTrap(Color.Gray);
					Api.PostEntry(this.ProcessFragmentId, 15, Color.Gray);
					// poruka otisla, obrisi post na homepageu (???!)
				}
			}
		}

		private string GetShortestPath(string message1, string message2)
		{
			if (message1 == "" && message2 != "")
				return message2;
			else if (message1 != "" && message2 == "")
				return message1;
			else if (message1 == "" && message2 == "")
				return "";

			string path1 = ExtractPath(message1);
			string path2 = ExtractPath(message2);

			int count1 = PathRoadCount(path1);
			int count2 = PathRoadCount(path2);
			// pronadji najkraci put od 2 ponudjena

			if (count1 < count2)
				return path1;
			else
				return path2;
		}

		private uint getLastIDPathRoad(string pathroad)
		{
			int index = pathroad.LastIndexOf(",");
			if (index >= 0)
				return Convert.ToUInt32(pathroad.Substring(index + 1));
			else if (pathroad.Length > 0)
			{
				if (pathroad.EndsWith(";"))
					return Convert.ToUInt32(pathroad.Substring(0, pathroad.Length - 1));
				else
					return Convert.ToUInt32(pathroad);
			}
			else
				return uint.MaxValue;
		}

		private string ExtractPath(string message)
		{
			if (message.IndexOf("PATH=") < 0)
				return "";

			int startpath = message.IndexOf("PATH=") + "PATH=".Length;
			message = message.Substring(startpath);

			int endpath = message.IndexOf(';');
			message = message.Substring(0, endpath);

			return message;
		}


		private string ExtractRoad(string message)
		{
			if (message.IndexOf("ROAD=") < 0)
				return "";

			int startpath = message.IndexOf("ROAD=") + "ROAD=".Length;
			message = message.Substring(startpath);

			int endpath = message.IndexOf(';');
			message = message.Substring(0, endpath);

			return message;
		}

		private string ReversePath(string path)
		{
			List<string> l = new List<string>();
			while (path.Length > 0)
			{
				if (path[0] == ';')
					break;

				int stop = path.IndexOf(',');
				if (stop < 0)
				{
					stop = path.IndexOf(';');
				}
				if (stop < 0)
					stop = path.Length;
				string temp = path.Substring(0, stop);
				l.Add(temp);

				if (stop + 1 >= path.Length)
					path = "";
				else
					path = path.Substring(stop + 1);
			}

			path = "";
			for (int i = l.Count - 1; i >= 0; i--)
			{
				path = path + l[i] + ",";
			}
			// path = path.Substring(0, path.Length - 1) + ';';
			if (path == "")
				return "";

			path = path.Substring(0, path.Length - 1);

			return path;
		}

		private int PathRoadCount(string pathroad)
		{
			int count = 0;

			while (pathroad.Length > 0)
			{
				if (pathroad[0] == ';')
					break;

				int stop = pathroad.IndexOf(',');
				if (stop < 0)
				{
					stop = pathroad.IndexOf(';');
				}
				if (stop < 0)
					stop = pathroad.Length;

				count++;

				if (stop + 1 >= pathroad.Length)
					pathroad = "";
				else
					pathroad = pathroad.Substring(stop + 1);
			}

			return count;
		}

		private string RemoveThisFromPath(string path)
		{
			if (path.EndsWith(Api.GetComputerId().ToString() + ";"))
			{
				path = path.Substring(0, path.Length - ((string)(Api.GetComputerId().ToString() + ";")).Length);
				if (path.Length > 0 && !path.EndsWith(";"))
					path = path.Substring(0, path.Length - 1) + ';';

				return path;
			}
			else
			{
				return "";
			}
		}

		private ShadowZFaxPersonality getPersonality()
		{
			try
			{
				string post = (string)Api.GetPost(this.ProcessFragmentId, 0);
				if (post.StartsWith("ShadowZFax:Initiator"))
					return ShadowZFaxPersonality.Initiator;
				else if (post.StartsWith("ShadowZFax:Listener"))
					return ShadowZFaxPersonality.Listener;
				else
					return ShadowZFaxPersonality.Carrier;
			}
			catch
			{
				return ShadowZFaxPersonality.Carrier;
			}
		}

		private ShadowZFaxInitiatorState getInitiatorState()
		{
			try
			{
				string post = (string)Api.GetPost(this.ProcessFragmentId, 1);
				if (post.StartsWith("Initiator:BrandNew"))
					return ShadowZFaxInitiatorState.brandNew;
				else if (post.StartsWith("Initiator:Enlightment"))
					return ShadowZFaxInitiatorState.enlightment;
				else if (post.StartsWith("Initiator:Brightness"))
					return ShadowZFaxInitiatorState.brightness;
				else if (post.StartsWith("Initiator:Sending"))
					return ShadowZFaxInitiatorState.sending;
				else if (post.StartsWith("Initiator:Waiting"))
					return ShadowZFaxInitiatorState.waiting;
				else if (post.StartsWith("Initiator:Killing"))
					return ShadowZFaxInitiatorState.killing;
				else if (post.StartsWith("Initiator:Dead"))
					return ShadowZFaxInitiatorState.dead;
				else
					return ShadowZFaxInitiatorState.dead;
			}
			catch
			{
				return ShadowZFaxInitiatorState.dead;
			}
		}


		private ShadowZFaxListenerState getListenerState()
		{
			try
			{
				string post = (string)Api.GetPost(this.ProcessFragmentId, 1);
				if (post.StartsWith("Listener:BrandNew"))
					return ShadowZFaxListenerState.brandNew;
				else if (post.StartsWith("Listener:Enlightment"))
					return ShadowZFaxListenerState.enlightment;
				else if (post.StartsWith("Listener:Brightness"))
					return ShadowZFaxListenerState.brightness;
				else if (post.StartsWith("Listener:Waiting"))
					return ShadowZFaxListenerState.waiting;
				else if (post.StartsWith("Listener:Answering"))
					return ShadowZFaxListenerState.answering;
				else if (post.StartsWith("Listener:Dieing"))
					return ShadowZFaxListenerState.dieing;
				else
					return ShadowZFaxListenerState.dieing;
			}
			catch
			{
				return ShadowZFaxListenerState.dieing;
			}
		}

		private ShadowZFaxCarrierState getCarrierState()
		{
			try
			{
				string post = (string)Api.GetPost(this.ProcessFragmentId, 1);
				if (post.StartsWith("Carrier:Setup"))
					return ShadowZFaxCarrierState.setup;
				else if (post.StartsWith("Carrier:PatchThrough"))
					return ShadowZFaxCarrierState.patchThrough;
				else if (post.StartsWith("Carrier:Gone"))
					return ShadowZFaxCarrierState.gone;
				else
					return ShadowZFaxCarrierState.setup;
			}
			catch
			{
				return ShadowZFaxCarrierState.setup;
			}
		}
	}
}