/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Remoting.Services;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Remoting.Channels;

namespace PaintableSimulator.Server
{
	public class TrackingHandler : ITrackingHandler
	{
		// Notify a handler that an object has been marshaled.
		public void MarshaledObject(Object obj, ObjRef or)
		{
			// Assumption: We have a server channel sink that sets a call context flag
			// called "ObservedIP" whenever we are processing a remote request
			object ObservedIP = CallContext.GetData("ObservedIP");

			// for local clients we don't do anything here
			// if they don't specify the remote IP then we just use the servers IP
			if (ObservedIP == null)
				return;

			if (or.ChannelInfo == null)
				return;

			string strAddress = (string)ObservedIP;

			for (int i = or.ChannelInfo.ChannelData.GetLowerBound(0);
							i <= or.ChannelInfo.ChannelData.GetUpperBound(0); i++)
			{
				// Check for the ChannelDataStore object that we don't want to copy
				if (or.ChannelInfo.ChannelData[i] is ChannelDataStore)
				{
					// Personally I don't know why ChannelURIs is an array... I am only
					// familiar with there being one URI in each ChannelDataStore object
					foreach (string uri in ((ChannelDataStore)or.ChannelInfo.ChannelData[i]).ChannelUris)
					{
						// this will get the first part of the uri
						int nOffset = uri.IndexOf("//") + 2;

						string strNewUri = uri.Substring(0, nOffset);
						strNewUri += strAddress;
						nOffset = uri.IndexOf(":", nOffset);
						strNewUri += uri.Substring(nOffset, uri.Length - nOffset);
						string[] strarray = new string[1] { strNewUri };

						ChannelDataStore cds = new ChannelDataStore(strarray);
						or.ChannelInfo.ChannelData[i] = cds;
					}
				}
			}
		}

		#region ITrackingHandler Members

		public void DisconnectedObject(object obj)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		public void UnmarshaledObject(object obj, ObjRef or)
		{
			throw new Exception("The method or operation is not implemented.");
		}

		#endregion
	}
}
