/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Collections.Generic;
using System.Text;
using PaintableSimulator.Engine;
using System.Diagnostics;
using System.Runtime.Remoting;
using System.Net.Sockets;
using PaintableSimulator.SmartClient.Resources;

namespace PaintableSimulator.SmartClient
{
	/// <summary>
	/// 
	/// </summary>
	public partial class SimulatorClient
	{

		private uint lastTickCount;
		private Stopwatch lastTickStopwatch;

		/// <summary>
		/// Initializes the info display.
		/// </summary>
		public void InitializeInfoDisplay()
		{
			lastTickStopwatch = new Stopwatch();

		}


		/// <summary>
		/// Refreshes the simulation info.
		/// </summary>
		private void RefreshSimulationInfo()
		{
			try
			{

				if (Program.simulator != null)
				{
					// get simulator information
					SimulatorInfo info = null;
					try
					{
						info = Program.simulator.SimulatorStatus;
					}
					catch (RemotingException ex)
					{
						Program.HandleIrrecoverableRemotingException(ex);
					}
					catch (SocketException ex)
					{
						Program.HandleIrrecoverableRemotingException(ex);
					}
					catch (Exception ex)
					{
						if (Program.log.IsErrorEnabled)
							Program.log.Error(UserMessages.SimulatorInfoReadError, ex);
					}

					// display simulator info
					if (info != null)
					{

						// calculate speed
						lastTickStopwatch.Stop();
						double elapsedSeconds = (double)(lastTickStopwatch.ElapsedTicks + 1) / Stopwatch.Frequency;
						double deltaSimulatorTick = info.TickCount - lastTickCount;
						long speedTicksPerSecond = (long)(deltaSimulatorTick / elapsedSeconds);

						lastTickCount = info.TickCount;
						lastTickStopwatch.Reset();
						lastTickStopwatch.Start();

						// calculate lost package percentage
						double lost = (double)info.Communication.LostPacketCount / (info.Communication.TotalPacketCount + 0.1) * 100;

						// display data
						simulationClockInfo.Text = string.Format("Iteration (speed): {0} ({1} ips)", info.TickCount, speedTicksPerSecond);
						simulationPacketsInfo.Text = string.Format("Packets (lost%): {0} ({1:0.00}%)", info.Communication.PendingPacketCount, lost);
					}

				}
			}
			catch (RemotingException ex)
			{
				Program.HandleIrrecoverableRemotingException(ex);
				return;
			}
			catch (SocketException ex)
			{
				Program.HandleIrrecoverableRemotingException(ex);
				return;
			}
			catch
			{
			}


			// read computer info
			simulationProcessorInfo.Text = String.Format("CPU Usage: {0}", processorUsageCounter.NextValue().ToString("0.0"));
			simulationMemoryInfo.Text = String.Format("Memory Usage: {0}", memoryUsageCounter.NextValue().ToString("0.00"));

		}
	}
}
