/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Net.Sockets;
using System.Runtime.Remoting;
using System.Threading;
using System.Windows.Forms;
using PaintableSimulator.Engine;
using System.Drawing.Imaging;
using System.Xml;
using PaintableSimulator.Engine.PaintableComputers;
using System.Xml.Serialization;

namespace PaintableSimulator.SmartClient
{
	public partial class SimulatorClient : Form
	{

		Random rnd = new Random(DateTime.Now.Second);

		/// <summary>
		/// Synchronization purpouses
		/// </summary>
		private object displaySyncRoot = new Object();

		/// <summary>
		/// Synchronization purpouses
		/// </summary>
		private object debugSyncRoot = new Object();

		// button handlers
		EventHandler roundSelectorButton;
		EventHandler squareSelectorButton;
		EventHandler polygonSelectorButton;
		EventHandler remove10PercentButton;
		EventHandler remove30PercentButton;

		/// <summary>
		/// Initializes a new instance of the <see cref="T:SimulatorClient"/> class.
		/// </summary>
		public SimulatorClient()
		{
			engineSettings = LoadEngineSettings();
			
			InitializeComponent();
		}


		/// <summary>
		/// Handles the Load event of the SimulatorClient control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void SimulatorClient_Load(object sender, EventArgs e)
		{
			// initialize the info display
			InitializeInfoDisplay();

			// load process fragments
			InitializeProcessFragments();

			// initialize debugger
			InitializeDebugger();

			// initialize deletion selectors
			InitializeDeleteSelectors();
			roundSelectorButton = new EventHandler(this.roundEraserToolStripMenuItem_Click);
			squareSelectorButton = new EventHandler(this.rectangleEraserToolStripMenuItem_Click);
			polygonSelectorButton = new EventHandler(this.polygonalEraserToolStripMenuItem_Click);
			remove10PercentButton = new EventHandler(this.remove10Percent_Click);
			remove30PercentButton = new EventHandler(this.remove30Percent_Click);

			InitializeSimulatorControl();

			userInterfaceRefreshTimer.Enabled = true;

			// TEMP
			this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
			this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			this.SetStyle(ControlStyles.UserPaint, true);
			
			PaintableSimulator.SmartClient.Logging.TextBoxAppender.OutputTextBox = logLister;

		}

		/// <summary>
		/// Handles the Click event of the newSimulation control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void newSimulation_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.CreateNewSimulator);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Click event of the initializeSimulation control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void initializeSimulation_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.InitializeSimulator);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Click event of the runSimulation control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void runSimulation_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.RunSimulation);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}


		/// <summary>
		/// Handles the Click event of the pauseSimulation control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void pauseSimulation_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.StopSimulation);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Click event of the stepSimulation control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void stepSimulation_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.StepSimulation);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}


		#region GUI Homekeeping Logic

		private enum HousekeepingAction
		{
			CreateNewSimulator,
			InitializeSimulator,
			RunSimulation,
			StopSimulation,
			StepSimulation

		}

		private void PerformHousekeeping(HousekeepingAction action)
		{
			try
			{

				switch (action)
				{
					case HousekeepingAction.CreateNewSimulator:
						{
							// clear log
							logLister.Text = "";

							// stop worker
							try
							{
								simulationWorker.CancelAsync();
							}
							finally { }

							// clear up debuggers
							lock (debugSyncRoot)
							{
								DisableDebugger();
							}

							// disable the additional tab
							additionalTab.Enabled = false;

							// create a new simulator
							CreateNewSimulator();

							// prevent errors
							if (Program.simulator == null)
								this.Close();

							// disable display controls
							displayControl.Enabled = false;

							//disable scroll bars
							displayHorizontalScroll.Enabled = false;
							displayVerticalScroll.Enabled = false;

							//clear the loaded processfragment list
							homepageColorizers.Items.Clear();
							homepageColorizers.Text = "";
							ScanProcesFragments(processFragmentTree);

							paintableComputers.Enabled = true;

							// button logic
							initializeSimulation.Enabled = true;
							runSimulation.Enabled = false;
							pauseSimulation.Enabled = false;
							stepSimulation.Enabled = false;

							zoomSelection.SelectedItem = "100%";

							// clear up process fragment selection
							processFragmentProperties.SelectedObject = null;
							processFragmentInitialization.Visible = false;
							processFragmentTree.SelectedNode = null;

							//menu item logic
							initializeSimulatorToolStripMenuItem.Enabled = true;

							// fill the default computer selection combo
							paintableComputers.Items.Clear();
							computersList.Items.Clear();
							FriendlyType[] types = Program.simulator.Plugins.ValidPaintableComputers.ToArray();
							paintableComputers.Items.AddRange(types);
							computersList.Items.AddRange(types);

							// select the Generic PC by default
							paintableComputers.SelectedItem = new FriendlyType(typeof(GenericPaintableComputer));
							computersList.SelectedIndex = 0;


							// fill colorizers
							homepageColorizers.Items.Clear();
							homepageColorizers.Items.AddRange(Program.simulator.Plugins.ValidHomepageColorizers.ToArray());
							homepageColorizers.SelectedItem = null;

							// fill feeders
							feedersList.Items.Clear();
							feedersList.Items.AddRange(Program.simulator.Plugins.ValidEnvironmentDataFeeders.ToArray());
							feedersList.SelectedItem = null;

							break;
						}

					case HousekeepingAction.InitializeSimulator:
						{
							if ((Program.simulator == null) || (Program.simulator.SimulatorStatus.SimulatorState != SimulatorStates.Created))
								return;

							paintableComputers.Enabled = false;

							simulatorProperties.Enabled = false;

							//initialize the simulator
							InitializeSimulator();

							// get static data
							displayInfo.WallSize = Program.simulator.SimulatorStatus.Environment.WallSize;
							displayInfo.ClientSize = simulatorDisplay.ClientSize;
							displayInfo.ScaleFactor = 1.0f;
							displayInfo.Buffer = new Bitmap(displayInfo.ClientSize.Width, displayInfo.ClientSize.Height, PixelFormat.Format32bppArgb);
							completeComputerDataList = Program.simulator.ComputerList;

							// set scroll bars
							RecalculateScrollBarLimits();

							// simulation drawing
							while (simulationWorker.IsBusy) ;
							simulationWorker.RunWorkerAsync();

							// button logic
							initializeSimulation.Enabled = false;
							runSimulation.Enabled = true;

							// enable debugger controls
							selectComputerToDebug.Enabled = true;
							selectComputerToolStripMenuItem.Enabled = true;
							stopDebugger.Enabled = false;
							stopDebuggerToolStripMenuItem.Enabled = false;

							// enable process fragment selection
							processFragmentsGroup.Enabled = true;

							// enable display controls
							displayControl.Enabled = true;

							//menu item logic
							initializeSimulatorToolStripMenuItem.Enabled = false;
							runSimulationToolStripMenuItem.Enabled = true;

							// enable the additional tab
							additionalTab.Enabled = true;


							break;
						}

					case HousekeepingAction.RunSimulation:
						{

							RunSimulation();

							if (IsSimulatorRunning())
							{
								// button logic
								initializeSimulation.Enabled = false;
								runSimulation.Enabled = false;
								pauseSimulation.Enabled = true;
								stepSimulation.Enabled = false;

								//menu item logic
								runSimulationToolStripMenuItem.Enabled = false;
								stopSimulationToolStripMenuItem.Enabled = true;
								stepSimulationToolStripMenuItem.Enabled = false;
							}

							break;
						}

					case HousekeepingAction.StopSimulation:
						{

							PauseSimulation();

							if (IsSimulatorPaused())
							{
								// button logic
								initializeSimulation.Enabled = false;
								runSimulation.Enabled = true;
								pauseSimulation.Enabled = false;
								stepSimulation.Enabled = true;


								//menu item logic
								runSimulationToolStripMenuItem.Enabled = true;
								stopSimulationToolStripMenuItem.Enabled = false;
								stepSimulationToolStripMenuItem.Enabled = true;
							}
							break;
						}

					case HousekeepingAction.StepSimulation:
						{
							StepSimulation();

							break;
						}


					default:
						break;
				}

			}
			catch (RemotingException ex)
			{
				Program.HandleIrrecoverableRemotingException(ex);
			}
			catch (SocketException ex)
			{
				Program.HandleIrrecoverableRemotingException(ex);
			}
			catch (Exception ex)
			{
				if (Program.log.IsErrorEnabled)
					Program.log.Error(ex);
			}

		}



		#endregion

		/// <summary>
		/// Handles the SelectedIndexChanged event of the zoomSelection control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void zoomSelection_SelectedIndexChanged(object sender, EventArgs e)
		{
			displayInfo.ScaleFactor = ScaleFactor();

			if (!IsSimulatorDataAvailable())
				return;

			RecalculateScrollBarLimits();
		}

		/// <summary>
		/// Handles the TextUpdate event of the zoomSelection control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void zoomSelection_TextUpdate(object sender, EventArgs e)
		{
			displayInfo.ScaleFactor = ScaleFactor();

			if (!IsSimulatorDataAvailable())
				return;

			RecalculateScrollBarLimits();
		}

		/// <summary>
		/// Handles the Tick event of the userInterfaceRefreshTimer control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void userInterfaceRefreshTimer_Tick(object sender, EventArgs e)
		{
			RefreshSimulationInfo();
		}

		/// <summary>
		/// Handles the SelectedIndexChanged event of the homepageColorizer control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void homepageColorizer_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (homepageColorizers.SelectedItem != null)
				displayInfo.ColorizerType = (FriendlyType)homepageColorizers.SelectedItem;
			else
				displayInfo.ColorizerType = null;
		}

		/// <summary>
		/// Handles the FormClosing event of the SimulatorClient control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.FormClosingEventArgs"/> instance containing the event data.</param>
		private void SimulatorClient_FormClosing(object sender, FormClosingEventArgs e)
		{
			try
			{
				PauseSimulation();

				simulationWorker.CancelAsync();
				while (simulationWorker.IsBusy)
				{
					Application.DoEvents();
				}

				userInterfaceRefreshTimer.Enabled = false;
				
				


				if (IsSimulatorRunning())
				{
					this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
					PerformHousekeeping(HousekeepingAction.StopSimulation);
					this.Cursor = System.Windows.Forms.Cursors.Default;
				}


			}
			finally
			{
			}
		}

		/// <summary>
		/// Handles the Click event of the newSimulatorToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void newSimulatorToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.CreateNewSimulator);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Click event of the initializeSimulatorToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void initializeSimulatorToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.InitializeSimulator);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Click event of the runSimulationToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void runSimulationToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.RunSimulation);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Click event of the stopSimulationToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void stopSimulationToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.StopSimulation);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Click event of the stepSimulationToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void stepSimulationToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
			PerformHousekeeping(HousekeepingAction.StepSimulation);
			this.Cursor = System.Windows.Forms.Cursors.Default;
		}

		/// <summary>
		/// Handles the Paint event of the simulatorDisplay control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.PaintEventArgs"/> instance containing the event data.</param>
		private void simulatorDisplay_Paint(object sender, PaintEventArgs e)
		{
			// force a repaint of the display
			try
			{
				lock (displaySyncRoot)
				{
					DrawVisualisationData(e.Graphics);
					DrawDebugData(e.Graphics);
					DrawHighlightedComputer(e.Graphics);
					DrawSelector(e.Graphics);
				}
			}
			finally { }
		}

		/// <summary>
		/// Handles the Click event of the showLeftPanelToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void showLeftPanelToolStripMenuItem_Click(object sender, EventArgs e)
		{
			collapsibleSplitter.ToggleState();
			showLeftPanelToolStripMenuItem.Checked = !collapsibleSplitter.IsCollapsed;
		}


		/// <summary>
		/// Handles the ToggleChanged event of the collapsibleSplitter control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void collapsibleSplitter_ToggleChanged(object sender, EventArgs e)
		{
			showLeftPanelToolStripMenuItem.Checked = !collapsibleSplitter.IsCollapsed;
		}

		/// <summary>
		/// Handles the Click event of the lockColorizer control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void lockColorizer_Click(object sender, EventArgs e)
		{
			lockColorizer.Checked = !lockColorizer.Checked;
		}

		/// <summary>
		/// Handles the KeyDown event of the SimulatorClient control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.Windows.Forms.KeyEventArgs"/> instance containing the event data.</param>
		private void SimulatorClient_KeyDown(object sender, KeyEventArgs e)
		{

			// handle some of the shortcut keys
			if (e.Control && (e.KeyCode == Keys.F))
			{
				simulationControlTabs.SelectedIndex = 1;
				e.SuppressKeyPress = true;
			}
			else if (e.Control && (e.KeyCode == Keys.D))
			{
				simulationControlTabs.SelectedIndex = 2;
				e.SuppressKeyPress = true;
			}
		}

		/// <summary>
		/// Handles the Click event of the polygonalEraserToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void polygonalEraserToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if ((sender == deleteComputers) && (deleteComputers.DropDownButtonPressed))
				return;

			// set tooltip
			genericToolTip.SetToolTip(simulatorDisplay, "Left click to select polygon points, right click to delete the computers");

			// disable any debuggers that are active
			DisableDebugger();

			if (IsSimulatorRunning())
				PerformHousekeeping(HousekeepingAction.StopSimulation);

			EnablePolygonSelector();

			// Register this action on the X button
			// disable all others
			deleteComputers.Click -= polygonSelectorButton;
			deleteComputers.Click -= squareSelectorButton;
			deleteComputers.Click -= roundSelectorButton;
			deleteComputers.Click -= remove10PercentButton;
			deleteComputers.Click -= remove30PercentButton;

			// enable the one we need
			deleteComputers.Click += polygonSelectorButton;
			

		}

		/// <summary>
		/// Handles the Click event of the roundEraserToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void roundEraserToolStripMenuItem_Click(object sender, EventArgs e)
		{
			if ((sender == deleteComputers) && (deleteComputers.DropDownButtonPressed))
				return;

			// set tooltip
			genericToolTip.SetToolTip(simulatorDisplay, "Press CTRL+UP to increase, or CTRL+DOWN to decrease the circle");

			// disable any debuggers that are active
			DisableDebugger();

			if (IsSimulatorRunning())
				PerformHousekeeping(HousekeepingAction.StopSimulation);

			EnableRoundSelector();

			// Register this action on the X button
			// disable all others
			deleteComputers.Click -= polygonSelectorButton;
			deleteComputers.Click -= squareSelectorButton;
			deleteComputers.Click -= roundSelectorButton;
			deleteComputers.Click -= remove10PercentButton;
			deleteComputers.Click -= remove30PercentButton;

			// enable the one we need
			deleteComputers.Click += roundSelectorButton;
		}

		/// <summary>
		/// Handles the Click event of the rectangleEraserToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void rectangleEraserToolStripMenuItem_Click(object sender, EventArgs e)
		{

			if ((sender == deleteComputers) && (deleteComputers.DropDownButtonPressed))
				return;

			// set tooltip
			genericToolTip.SetToolTip(simulatorDisplay, "Click and drag");

			// disable any debuggers that are active
			DisableDebugger();

			if (IsSimulatorRunning())
				PerformHousekeeping(HousekeepingAction.StopSimulation);

			EnableSquareSelector();


			// Register this action on the X button
			// disable all others
			deleteComputers.Click -= polygonSelectorButton;
			deleteComputers.Click -= squareSelectorButton;
			deleteComputers.Click -= roundSelectorButton;
			deleteComputers.Click -= remove10PercentButton;
			deleteComputers.Click -= remove30PercentButton;

			// enable the one we need
			deleteComputers.Click += squareSelectorButton;

		}



		/// <summary>
		/// Handles the Click event of the remove10Percent control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void remove10Percent_Click(object sender, EventArgs e)
		{
			if ((sender == deleteComputers) && (deleteComputers.DropDownButtonPressed))
				return;

			// disable any debuggers that are active
			DisableDebugger();

			// pause the simulation
			if (IsSimulatorRunning())
				PerformHousekeeping(HousekeepingAction.StopSimulation);

			// Register this action on the X button
			// disable all others
			deleteComputers.Click -= polygonSelectorButton;
			deleteComputers.Click -= squareSelectorButton;
			deleteComputers.Click -= roundSelectorButton;
			deleteComputers.Click -= remove10PercentButton;
			deleteComputers.Click -= remove30PercentButton;

			// enable the one we need
			deleteComputers.Click += remove10PercentButton;


			int removeCount = (int)(completeComputerDataList.Count * 0.1);
			List<int> indices = new List<int>();

			// randomly select the indices
			for (int i = 0; i < removeCount; i++)
			{
				int index = rnd.Next(completeComputerDataList.Count - 1);

				if (indices.Contains(index))
					i--;
				else
					indices.Add(index);

			}

			// disable the 10% computers
			Program.simulator.DisableComputers(indices, true);

			if (Program.EnableRemoting)
				completeComputerDataList = Program.simulator.ComputerList;
		}

		/// <summary>
		/// Handles the Click event of the remove30Percent control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void remove30Percent_Click(object sender, EventArgs e)
		{
			if ((sender == deleteComputers) && (deleteComputers.DropDownButtonPressed))
				return;

			// disable any debuggers that are active
			DisableDebugger();

			// pause the simulation
			if (IsSimulatorRunning())
				PerformHousekeeping(HousekeepingAction.StopSimulation);

			// Register this action on the X button
			// disable all others
			deleteComputers.Click -= polygonSelectorButton;
			deleteComputers.Click -= squareSelectorButton;
			deleteComputers.Click -= roundSelectorButton;
			deleteComputers.Click -= remove10PercentButton;
			deleteComputers.Click -= remove30PercentButton;

			// enable the one we need
			deleteComputers.Click += remove30PercentButton;


			int removeCount = (int)(completeComputerDataList.Count * 0.3);
			List<int> indices = new List<int>();

			// randomly select the indices
			for (int i = 0; i < removeCount; i++)
			{
				int index = rnd.Next(completeComputerDataList.Count - 1);

				if (indices.Contains(index))
					i--;
				else
					indices.Add(index);

			}

			// disable the 10% computers
			Program.simulator.DisableComputers(indices, true);

			if (Program.EnableRemoting)
				completeComputerDataList = Program.simulator.ComputerList;

		}


		/// <summary>
		/// Handles the Click event of the registerEnvironmentFeeder control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void registerEnvironmentFeeder_Click(object sender, EventArgs e)
		{
			if ((IsSimulatorDataAvailable()) && (feedersList.SelectedItem != null))
			{
				// pause if running
				if (IsSimulatorRunning())
					PerformHousekeeping(HousekeepingAction.StopSimulation);

				try
				{
					// register the feeder
					Program.simulator.RegisterEnvironmentDataFeed((FriendlyType)feedersList.SelectedItem);
				}
				catch
				{
					MessageBox.Show("Failed");
				}

			}
		}

		/// <summary>
		/// Handles the Click event of the addComputer control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void addComputer_Click(object sender, EventArgs e)
		{
			if ((IsSimulatorDataAvailable()) && (computersList.SelectedItem != null))
			{
				// pause if running
				if (IsSimulatorRunning())
					PerformHousekeeping(HousekeepingAction.StopSimulation);

				simulatorDisplay.MouseDown += this.addComputerMouseClick;
				simulatorDisplay.Cursor = System.Windows.Forms.Cursors.Cross;

			}
		}

		/// <summary>
		/// Handles the Click event of the aboutToolStripMenuItem control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:System.EventArgs"/> instance containing the event data.</param>
		private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
		{
			AboutBox ab = new AboutBox();
			ab.ShowDialog();
		}

	}
}