/*
	Paintable Computing Simulation

	License

	This software (Paintable Computing Simulation and all parts of its 
	source code) is provided "as is", without any warranty and it is 
	the property of its respective authors. Use of this software is 
	free to everyone, except in commercial and/or military purposes 
	without written permission of its authors. Disassembling the 
	executable code of this software is forbidden for any purpose 
	without written permission of its authors.

	If you intend to use any part of the source code of this software 
	or to develop this software any further, you must include this 
	license in every source file and every About box, without changing 
	it; only appending the license is allowed. And if you use any part 
	of the source code of this software, or develop it further, please 
	send an e-mail to its authors.

	Authors of this software are not responsible for any consequences 
	which use of this software, use of any part of its source or its 
	further development may cause.

	All software and parts of source code used in the development of 
	this software are property of their respective owners.

	Good luck,
	Authors

	Authors (alphabetically):
	Fabijan Lukin
		fabijan@gmail.com
		fabijan.lukin@fer.hr
	Fran Pregernik
		fran.pregernik@gmail.com
		fran.pregernik@fer.hr
	Tomislav Sukser
		tomislav.sukser@gmail.com
		tomislav.sukser@zg.t-com.hr
		tomislav.sukser@fer.hr


	Note:
	Paintable computing was firstly introduced in Programming a 
	Paintable Computer written by William Joseph Butera.


    Copyright (c) 2005,2006. All rights reserved.
    4-6/2006

 */
using System;
using System.Net;
using System.Net.NetworkInformation;
using System.Windows.Forms;
using Gui.Wizard;

namespace PaintableSimulator.SmartClient
{
	public partial class StartupWizard : Form
	{
		/// <summary>
		/// 
		/// </summary>
		public StartupWizard()
		{
			InitializeComponent();

				// get defaults
				hostName.Text = Program.ServerIP.ToString();
				serverPort.Text = Program.ServerPort.ToString();
				serverApplicationName.Text = Program.ServerApplicationName;

				//hide test log
				testLog.Text = "";
				testLog.Visible = false;


		}

		/// <summary>
		/// Handles the CloseFromNext event of the remoteSimulatorOptionsPage control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="T:Gui.Wizard.PageEventArgs"/> instance containing the event data.</param>
		private void remoteSimulatorOptionsPage_CloseFromNext(object sender, PageEventArgs e)
		{
				// store the server IP
				if (hostName.Text.Length != 0)
				{
					try
					{
						IPAddress[] ips = Dns.GetHostAddresses(hostName.Text);

						Program.ServerIP = ips[0];
					}
					catch (Exception ex)
					{
						MessageBox.Show("Server IP error: " + ex.Message);
						e.Page = remoteSimulatorOptionsPage;
					}
				}
				else
				{
					MessageBox.Show("Server IP error: Host not entered!");
					e.Page = remoteSimulatorOptionsPage;
					return;
				}


				// store the server port
				try
				{
					Program.ServerPort = uint.Parse(serverPort.Text);
				}
				catch (Exception ex)
				{
					MessageBox.Show("Server port error: " + ex.Message);
					e.Page = remoteSimulatorOptionsPage;
					return;
				}


				// store the application path
				if (serverApplicationName.Text.Length != 0)
					Program.ServerApplicationName = serverApplicationName.Text;
				else
				{
					MessageBox.Show("Server application name error: Name not entered");
					e.Page = remoteSimulatorOptionsPage;
					return;
				}


				finishInfoPage.PageText = "The simulator will be running on a server!\r\n\r\nThe URI is: ";
				finishInfoPage.PageText +=
					string.Format("http://{0}:{1}/{2}", Program.ServerIP.ToString(), Program.ServerPort.ToString(),
					              Program.ServerApplicationName);
			
		}


		private void testConfiguration_Click(object sender, EventArgs e)
		{
			IPAddress serverIP = IPAddress.None;
			uint port;
			string appName;

			// store the server IP
			if (hostName.Text.Length != 0)
			{
				try
				{
					IPAddress[] ips = Dns.GetHostAddresses(hostName.Text);

					serverIP = ips[0];
				}
				catch (Exception ex)
				{
					MessageBox.Show("Server IP error: " + ex.Message);
				}
			}
			else
			{
				MessageBox.Show("Server IP error: Host not entered!");
				return;
			}


			// store the server port
			try
			{
				port = uint.Parse(serverPort.Text);
			}
			catch (Exception ex)
			{
				MessageBox.Show("Server port error: " + ex.Message);
				return;
			}


			// store the application path
			if (serverApplicationName.Text.Length != 0)
				appName = serverApplicationName.Text;
			else
			{
				MessageBox.Show("Server application name error: Name not entered");
				return;
			}


			testLog.Text = "";
			testLog.Visible = true;

				AppDomain testDomain = AppDomain.CreateDomain("TestDomain");

				testLog.Text += String.Format("Trying to create a remote object...\r\n");

				// call the remoting tester
				testDomain.SetData("ServerIP", serverIP);
				testDomain.SetData("ServerPort", port);
				testDomain.SetData("ServerApplicationName", appName);
				testDomain.DoCallBack(new CrossAppDomainDelegate(RemotingTester.Test));

				// get tester result
				bool? result = (bool?) testDomain.GetData("RemotingResult");

				if (result == true)
				{
					testLog.Text += String.Format("Success! Remote object created.\r\n");
					testLog.Text += String.Format("\r\n\r\nConfiguration OK!");
				}
				else
					testLog.Text += String.Format("Failure! Check your port and application settings!\r\n");

				// unload the test domain
				AppDomain.Unload(testDomain);
				testDomain = null;

		}


	}
}