using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace PaintableSimulator.Engine
{

	/// <summary>
	/// 
	/// </summary>
	public class CommunicationInfo
	{

		internal uint totalPacketCount;
		/// <summary>
		/// Gets the total packet count.
		/// </summary>
		/// <value>The total packet count.</value>
		public uint TotalPacketCount
		{
			get { return totalPacketCount; }
		}

		internal uint lostPacketCount;
		/// <summary>
		/// Gets the lost packet count.
		/// </summary>
		/// <value>The lost packet count.</value>
		public uint LostPacketCount
		{
			get { return lostPacketCount; }
		}


		internal uint pendingPacketCount;
		/// <summary>
		/// Gets the pending packet count.
		/// </summary>
		/// <value>The pending packet count.</value>
		public uint PendingPacketCount
		{
			get { return pendingPacketCount; }
		}
	}

	/// <summary>
	/// 
	/// </summary>
	public class EnvironmentInfo
	{

		internal Size wallSize;
		/// <summary>
		/// Gets the size of the wall.
		/// </summary>
		/// <value>The size of the wall.</value>
		public Size WallSize
		{
			get { return wallSize; }
		}


		internal Size gridSize;
		/// <summary>
		/// Gets the size of the grid.
		/// </summary>
		/// <value>The size of the grid.</value>
		public Size GridSize
		{
			get { return gridSize; }
		}
	}


    /// <summary>
    /// Describes current state of simulator
    /// </summary>
    public enum SimulatorStates { None, Created, Initialized, ReadyToRun, Running, Stopped };
        
	/// <summary>
	/// 
	/// </summary>
	public class SimulatorInfo
	{

		internal uint computerCount;
		/// <summary>
		/// Gets the computer count.
		/// </summary>
		/// <value>The computer count.</value>
		public uint ComputerCount
		{
			get { return computerCount; }
		}

		internal uint tickCount;
		/// <summary>
		/// Gets the tick count.
		/// </summary>
		/// <value>The tick count.</value>
		public uint TickCount
		{
			get { return tickCount; }
		}

        internal SimulatorStates simulatorState;
        /// <summary>
        /// 
        /// </summary>
        public SimulatorStates SimulatorState
        {
            get { return simulatorState; }
        }
        

		private CommunicationInfo communication;
		/// <summary>
		/// Gets the communication.
		/// </summary>
		/// <value>The communication.</value>
		public CommunicationInfo Communication
		{
			get { return communication; }
		}


		private EnvironmentInfo environment;
		/// <summary>
		/// Gets or sets the environment.
		/// </summary>
		/// <value>The environment.</value>
		public EnvironmentInfo Environment
		{
			get { return environment; }
			set { environment = value; }
		}


		/// <summary>
		/// Initializes a new instance of the <see cref="T:SimulatorInfo"/> class.
		/// </summary>
		public SimulatorInfo()
		{
			communication = new CommunicationInfo();
			environment = new EnvironmentInfo();

            simulatorState = SimulatorStates.None;
		}

	}
}
